#!/usr/bin/env python
'''
Created on Oct 23, 2013

@author: paepcke

Finds all files in subdir 'data' that end with 'Truth.sql'.
Attempts to load them into MySql to check for errors.
Used to ensure that unittest truth files are legal MySQL
Assumed to be running in <json_to_relation-ProjectRoot>/scripts

'''
import os
import pymysql
import sys

from json_to_relation.mysqldb import MySQLDB


class TruthFileTester(object):
    
    def __init__(self, testing=False):
        if testing:
            return
        self.mysqlDb = MySQLDB(user='paepcke')
        
	scriptDir = os.path.dirname(os.path.abspath(__file__))
	print scriptDir #****
	truthFilesDir = os.path.join(scriptDir, '../json_to_relation/test/data')
	print truthFilesDir #****
        for filename in os.listdir(truthFilesDir):
            if filename.endswith('Truth.sql'):
                print('Testing truth file %s' % filename)
                for sqlStatement in self.getSQLStatement(os.path.join(truthFilesDir, filename)):
                    try:
                        for res in self.mysqlDb.query(sqlStatement):
                            print res
                    except (pymysql.err.InternalError, pymysql.err.IntegrityError, pymysql.err.Error) as e:
                        print >>sys.stderr, 'Failed in filename %s: %s' % (filename, `e`)
        print 'Done.'

    def getSQLStatement(self, fileName):
        fullLine = ''
        for line in open(fileName, 'r'):
            line = line.strip()
            fullLine += line
            if fullLine.endswith(';'):
                yield fullLine
                fullLine = ''
             
if __name__ == '__main__':
    
    TruthFileTester(False) # turn off test mode
#     tester = TruthFileTester(True) # turn on test mode
#     for line in tester.getSQLStatement('data/problemCheckInPathTruth.sql'):
#         if line is None:
#             print('done')
#             sys.exit()
#         else:
#             print(line)
            



'''
Created on Apr 7, 2014

@author: paepcke

NOTE: Requires existence of database 'unittest' and user 'unittest' without pwd
       and ALL privileges. (Could get away with fewer privileges, but who cares. 

'''
from collections import OrderedDict
import datetime
import unittest

from scripts.addAnonToActivityGradeTable import AnonAndModIDAdder
from pymysql_utils.pymysql_utils import MySQLDB


class TestAddAnonToActivityGrade(unittest.TestCase):

    studentmoduleExcerptSchema = OrderedDict({
                'activity_grade_id' : 'INT',
                'student_id' : 'INT',
                'course_display_name' : 'VARCHAR(255)',
                'grade' : 'VARCHAR(5)',
                'max_grade' : 'DOUBLE',
                'percent_grade' : 'DOUBLE',
                'parts_correctness' : 'VARCHAR(255)',
                'answers' : 'VARCHAR(255)',
                'num_attempts' : 'INT',
                'first_submit' : 'DATETIME',
                'last_submit' : 'DATETIME',
                'module_type' : 'VARCHAR(255)',
                'anon_screen_name' : 'VARCHAR(40)',
                'resource_display_name' : 'VARCHAR(255)',
                'module_id' : 'VARCHAR(255)'
                })
    
    studentmoduleExcerptColNames = [
                'activity_grade_id',
                'student_id',
                'course_display_name',
                'grade',
                'max_grade',
                'percent_grade',
                'parts_correctness',
                'answers',
                'num_attempts',
                'first_submit',
                'last_submit',
                'module_type',
                'anon_screen_name',
                'resource_display_name',
                'module_id'
                ]
    state1 = ' {"correct_map": {"i4x-Medicine-HRP258-problem-0c6cf38317be42e0829d10cc68e7451b_2_1": {"hint": "", "hintmode": null, "correctness": "correct", "npoints": null, "msg": "", "queuestate": null}}, "input_state": {"i4x-Medicine-HRP258-problem-0c6cf38317be42e0829d10cc68e7451b_2_1": {}}, "attempts": 1, "seed": 1, "done": true, "student_answers": {"i4x-Medicine-HRP258-problem-0c6cf38317be42e0829d10cc68e7451b_2_1": "choice_1"}} '
    state2 = '{"correct_map": {}, "seed": 1, "student_answers": {}, "input_state": {"i4x-Medicine-HRP258-problem-0c6cf38317be42e0829d10cc68e7451b_2_1": {}}}'
    state3 = '{"position": 1}'
    
    modid1 = 'i4x://Carnegie/2013/chapter/1fee4bc0d5384cb4aa7a0d65f3ac5d9b'
    modid2 = 'i4x://Carnegie/2013/chapter/5d08d2bae3ac4047bf5abe1d8dd16ac3'
    modid3 = 'i4x://Carnegie/2013/chapter/9a9455cd30bd4c14819542bcd11bfcf8'
    studentmoduleExcerptValues = \
                [
                [0,1,'myCourse',3,10,-1.0,state1,'',-1,'2014-01-10 04:10:45','2014-02-10 10:14:40','modtype1','','',modid1],
                [1,2,'myCourse',5,10,-1.0,state2,'',-1,'2014-01-10 11:30:23','2014-02-10 14:30:12','modtype2','','',modid2],                
                [2,3,'yourCourse',8,10,-1.0,state3,'',-1,'2014-01-10 18:34:12','2014-02-10 19:10:33','modtype2','','',modid3]                                
               ]

    def setUp(self):
        self.allColNames = TestAddAnonToActivityGrade.studentmoduleExcerptColNames[0]
        for colName in  TestAddAnonToActivityGrade.studentmoduleExcerptColNames[1:]:
            self.allColNames += ',' + colName
        
        self.db = MySQLDB(user='unittest', passwd='', db='unittest')
        self.db.dropTable('StudentmoduleExcerpt')
        self.db.createTable('StudentmoduleExcerpt', 
                            TestAddAnonToActivityGrade.studentmoduleExcerptSchema,
                            temporary=False)
                            #***temporary=True)
        self.db.bulkInsert('StudentmoduleExcerpt',
                           TestAddAnonToActivityGrade.studentmoduleExcerptColNames,
                           TestAddAnonToActivityGrade.studentmoduleExcerptValues)
        self.db.createTable('ActivityGrade', TestAddAnonToActivityGrade.studentmoduleExcerptSchema)
        # Make sure there isn't left over content (if the table existed):
        self.db.truncateTable('ActivityGrade')
        self.db.close()
    def tearDown(self):
        self.db = MySQLDB(user='unittest', passwd='', db='unittest')
        # Can't drop tables: hangs
        #self.db.dropTable('StudentmoduleExcerpt')
        #self.db.dropTable('ActivityGrade')
        self.db.close()
        pass
        
        
    def testAddAnonToActivityTable(self):
        try:
            # Modify the fake courseware_studentmodule excerpt
            # to add anon_screen_name, computer plusses/minusses, 
            # compute grade percentage, etc:
            AnonAndModIDAdder('unittest', '', db='unittest')
            self.db = MySQLDB(user='unittest', passwd='', db='unittest')
            for rowNum, row in enumerate(self.db.query('SELECT %s FROM ActivityGrade;' % self.allColNames)):
                #print(row)
                if rowNum == 0:
                    self.assertEqual((0, 1, 'myCourse', '3', 10.0, 30.0, '', '', -1, datetime.datetime(2014, 1, 10, 4, 10, 45), datetime.datetime(2014, 2, 10, 10, 14, 40), 'modtype1', 'None', 'Guided Walkthrough', 'i4x://Carnegie/2013/chapter/1fee4bc0d5384cb4aa7a0d65f3ac5d9b'), 
                                     row)
                elif rowNum == 1:
                    self.assertEqual((1, 2, 'myCourse', '5', 10.0, 50.0, '', '', -1, datetime.datetime(2014, 1, 10, 11, 30, 23), datetime.datetime(2014, 2, 10, 14, 30, 12), 'modtype2', 'None', 'Evaluation', 'i4x://Carnegie/2013/chapter/5d08d2bae3ac4047bf5abe1d8dd16ac3'),
                                     row)
                elif rowNum == 2:
                    self.assertEqual((2, 3, 'yourCourse', '8', 10.0, 80.0, '', '', -1, datetime.datetime(2014, 1, 10, 18, 34, 12), datetime.datetime(2014, 2, 10, 19, 10, 33), 'modtype2', 'None', 'Introduction', 'i4x://Carnegie/2013/chapter/9a9455cd30bd4c14819542bcd11bfcf8'),
                                     row)         
        finally:
            self.db.close()
        

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testAddAnonToActivityTable']
    unittest.main()



'''
Created on Nov 3, 2013

Manages fast lookup of countries, given words that *might* be countries.


@author: paepcke
'''
import os


class LocationManager(object):
    '''
    Manages fast lookup of countries, given words that *might* be countries.
    '''


    def __init__(self):
        '''
        Prepare the first-two-letter country dict.
        '''
        # Dict mapping first two letters of countries to list
        # of countries that start with those two letters:
        self.countryLookup = {}
        
        countryFile = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/countries.txt')
        with open(countryFile, 'r') as fd:
            allCountries = fd.readlines()
        for country in allCountries:
            country = country.strip()
            existingList = self.countryLookup.get(country[:2].upper(), None)
            if existingList is None:
                self.countryLookup[country[:2].upper()] = [country]
            else:
                self.countryLookup[country[:2].upper()].append(country)
                
    def isCountry(self, candidateStr):
        countryList = self.countryLookup.get(candidateStr[:2].upper(), None)
        if countryList is None:
            return ''
        candidateStr = candidateStr[0].upper() + candidateStr[1:]
        for country in countryList:
            if candidateStr == country:
                if country == 'US' or\
                   country == 'United States' or\
                   country == 'United States of America':
                    country = 'USA'
                return country
        return ''



#!/usr/bin/env python
'''
Created on Dec 22, 2013

@author: paepcke
'''
import os
import re
import sys

from edxTrackLogJSONParser import EdXTrackLogJSONParser
from modulestoreImporter import ModulestoreImporter
from unidecode import unidecode

idExtractPat = re.compile(r'^"([^"]*)')
seqIDExtractPat = re.compile(r'","([^"]*)')


hashLookup = ModulestoreImporter(os.path.join(os.path.dirname(__file__),'data/modulestore_latest.json'), 
                   useCache=True)

def makeInsertSafe(unsafeStr):
    '''
    Makes the given string safe for use as a value in a MySQL INSERT
    statement. Looks for embedded CR or LFs, and turns them into 
    semicolons. Escapes commas and single quotes. Backslash is
    replaced by double backslash. This is needed for unicode, like
    \0245 (invented example)
    @param unsafeStr: string that possibly contains unsafe chars
    @type unsafeStr: String
    @return: same string, with unsafe chars properly replaced or escaped
    @rtype: String
    '''
    #return unsafeStr.replace("'", "\\'").replace('\n', "; ").replace('\r', "; ").replace(',', "\\,").replace('\\', '\\\\')
    if unsafeStr is None or not isinstance(unsafeStr, basestring) or len(unsafeStr) == 0:
        return ''
    # Check for chars > 128 (illegal for standard ASCII):
    for oneChar in unsafeStr:
        if ord(oneChar) > 128:
            # unidecode() replaces unicode with approximations. 
            # I tried all sorts of escapes, and nothing worked
            # for all cases, except this:
            unsafeStr = unidecode(unicode(unsafeStr))
            break
    return unsafeStr.replace('\n', "; ").replace('\r', "; ").replace('\\', '').replace("'", r"\'")


def fixSequencIDs():
    counter = 0
    with open('/home/paepcke/tmp/sequenceIDs.sql','w') as outfd:
        outfd.write("USE Edx;\nINSERT INTO EdxTrackEvent(_id,resource_display_name)\n")
        with open('/home/paepcke/tmp/sequenceIDs.csv','r') as fd:
            for idSeqID in fd:
                sqlid    = idExtractPat.search(idSeqID).group(1) 
                seqID = seqIDExtractPat.search(idSeqID).group(1)
                resourceNameMatch = EdXTrackLogJSONParser.findHashPattern.search(seqID)
                if resourceNameMatch is not None:
                    resourceName = makeInsertSafe(hashLookup.getDisplayName(resourceNameMatch.group(1)))
                    if counter == 0:
                        outfd.write('("%s","%s")' % (sqlid,resourceName))
                    else:
                        outfd.write(',\n("%s","%s")' % (sqlid,resourceName))
                else:
                    continue
                counter += 1
                #if counter > 10:
                #    break
        outfd.write("\nON DUPLICATE KEY UPDATE resource_display_name = VALUES(resource_display_name);\n")
        print("Created %d corrections." % counter)
if __name__ == '__main__':
    fixSequencIDs()
    
    
#INSERT INTO EdxTrackEvent (_id,long_answer) VALUES ('fbcefe06_fb7c_48aa_a12e_d85e6988dbda','first answer'),('bbd3ddf3_8ed0_4eee_8ff7_f5791b9e4a7e','second answer') ON DUPLICATE KEY UPDATE long_answer=VALUES(long_answer);




'''
Created on Sep 23, 2013

@author: paepcke
'''
import sys


TINY_TEXT_LEN = 2**8
TEXT_LEN = 2**16
MEDIUM_TEXT_LEN = 2**24
LONG_TEXT_LEN = 2**32

TINY_INT_MAX = 2**8 - 1
TINY_INT_MIN = -2**7

SMALL_INT_MAX = 2**16 - 1
SMALL_INT_MIN = 2**15

MEDIUM_INT_MAX = 2**24 - 1
MEDIUM_INT_MIN = -2**23

INT_MAX = 2**32 - 1
INT_MIN = -2**31

BIG_INT_MAX = 2**64 - 1
BIG_INT_MIN = -2**63

FLOAT_MAX = sys.float_info.max
FLOAT_MIN = sys.float_info.min

class ColDataType:
    '''
    Enum for datatypes that can be converted to 
    MySQL datatypes
    '''
    TINYTEXT=0     # length < 2^8
    TEXT=1         # length < 2^16
    MEDIUMTEXT=2   # length < 2^24
    LONGTEXT=3     # length < 2^32
    TINYINT=4      # 1 byte
    SMALLINT=5     # 2 bytes
    MEDIUMINT=6    # 3 bytes
    INT=7          # 4 bytes
    BIGINT=8       # 8 bytes
    FLOAT=9        # 4 bytes
    DOUBLE=10      # 8 bytes
    DATE=11
    TIME=12
    DATETIME=13
    BOOL=14        # IF YOU ADD ENTRIES, MODIFY isistance method below
    UUID=15        # 32 byte key via uuid.uuid4()
        
    strings = {TINYTEXT : "VARCHAR(255)",
               TEXT     : "TEXT",
               MEDIUMTEXT : "MEDIUMTEXT",
               LONGTEXT : "LONGTEXT",
               TINYINT  : "TINYINT",
               SMALLINT : "SMALLINT",
               MEDIUMINT: "MEDIUMINT",
               INT      : "INT",
               BIGINT   : "BIGINT",
               FLOAT    : "FLOAT",
               DOUBLE   : "DOUBLE",
               DATE     : "DATE",
               TIME     : "TIME",
               DATETIME : "DATETIME",
               BOOL     : "BOOL",
               UUID     : "VARCHAR(40)"
    }
    
    defaultValues = {TINYTEXT : '',
                	 TEXT     : '',
                	 MEDIUMTEXT : '',
                	 LONGTEXT : '',
                	 TINYINT  : -1,
                	 SMALLINT : -1,
                	 INT      : -1,
                	 BIGINT   : -1,
                	 FLOAT    : -1.0,
                	 DOUBLE   : -1.0,
                	 DATE     : '00000000',
                	 TIME     : '000000',
                	 DATETIME : '00000000000000',
                	 BOOL     : 0,
                	 UUID     : ''
                     }

    @classmethod
    def isinstance(cls, value):
        return value >= ColDataType.TINYTEXT and value <= ColDataType.UUID

    def toString(self, val):
        try:
            return ColDataType.strings[val]
        except KeyError:
            raise ValueError("The code %s does not refer to a known datatype." % str(val))
        
    @classmethod
    def sqlTypeFromValue(cls, value):
        '''
        Given a value, return the best fitting MySQL data type
        Special case: None: we return TEXT.

        :param cls: ColDataType, makes it a class method
        :type cls: ColDataType
        :param value: example value
        :type value: {str | unicode | int | float | bool | None}
        '''

        if value is None:
            # Tough one: we'll guess that non-null values will
            # be text. Arbitrary choice:
            return ColDataType.TEXT
        
        if isinstance(value, bool):
            return ColDataType.BOOL
        
        if isinstance(value, str) or isinstance(value, unicode):
            if len(value) < TINY_TEXT_LEN:
                return ColDataType.TINYTEXT
            elif len(value) < MEDIUM_TEXT_LEN:
                return ColDataType.MEDIUMTEXT
            else:
                return ColDataType.LONGTEXT
            
        elif isinstance(value, int):
            if value < TINY_INT_MAX and value >= TINY_INT_MIN:
                return ColDataType.TINYINT
            elif value < MEDIUM_INT_MAX and value >= MEDIUM_INT_MIN:
                return ColDataType.MEDIUMINT
            elif value < INT_MAX and value >= INT_MIN:
                return ColDataType.INT
            else:
                return ColDataType.BIGINT
            
        elif isinstance(value, float):
            if value < FLOAT_MAX and value >= FLOAT_MIN:
                return ColDataType.FLOAT
            else:
                return ColDataType.DOUBLE
        else:
            raise ValueError("Unknown type for value '%s'" % str(value) )
        



'''
Very simple Python interface to MongoDB.

@Author: paepcke
'''

from pymongo import MongoClient


class MongoDB(object):
    '''
    Very simple Python interface to MongoDB. Based on pymongo,
    this class provides methods to get and set default databases
    and collections, to insert documents, query collections, and
    clear all documents in a collection. 
    
    The query() method encapsulates MangoDB native methods find() 
    and find_one(). The query() method makes it very convenient to
    to request only particular sets of fields (columns in relational 
    terms). Example::
    
        myMongoDb.query({'lname' : 'Doe'}, ('fname', 'lname', 'age'))
        
    '''
    
    # Map of query strings to MongoDB cursors. Administered
    # by query() method. Used in resultCount():
    queryCursors = {}
    
    # ----------------------------  Public Methods -------------------
    
    def __init__(self, host="localhost", ssl_keyfile=None, dbName="test", collection="test_collection", port=27017, user=None, pwd=""):
        '''
        Create a connection to the MongoDB demon on the given host/port.

        :param host: host name where MongoDB demon is running. Can be IP address as string.
        :type host: String
        :param port: MongoDB demon's port
        :type port: int
        '''
        self.host       = host
        self.port       = port
        self.dbName     = dbName
        self.collection = collection
        if user is not None:
            # Use Mongodb URI, mongodb://[username:password@]host1[:port1][,host2[:port2],...[,hostN[:portN]]][/[database][?options]] 
            # e.g.: mongodb://readonly:xxxxx@stanford-edx-prod.m0.mongolayer.com/stanford-edx-prod)
            if ssl_keyfile is None:
                self.client=MongoClient("mongodb://%s:%s@%s/%s" % (user,pwd,host,dbName), port)
            else:
                self.client=MongoClient("mongodb://%s:%s@%s/%s" % (user,pwd,host,dbName), port, ssl_keyfile=ssl_keyfile)
        else:
            if ssl_keyfile is None:
                self.client=MongoClient("mongodb://%s/%s" % (host,dbName), port)
            else:
                self.client=MongoClient("mongodb://%s/%s" % (host,dbName), port, ssl_keyfile=ssl_keyfile)
        self.setDB(dbName)
        self.setCollection(collection)
         
    def setDB(self, dbName):
        '''
        Establish a default database within MongoDB for subsequent calls 
        to other methods of this class.

        :param dbName: MongoDB database name
        :type dbName: String
        '''
        self.dbName = dbName
        self.db = self.client[dbName]
        
    def getDBName(self):
        '''
        Obtain the name of the MongoDB database that is
        currently the default for calls to methods of this class.

        :rtype: String
        '''
        return self.dbName

    def setCollection(self, collName):
        '''
        Establish a default MongoDB collection for subsequent calls to 
        other methods of this class.

        :param collName: MongoDB collection name
        :type collName: String
        '''
        self.coll = self.db[collName]
        
    def getCollectionName(self):
        '''
        Obtain the name of the MongoDB collection that is
        currently the default for calls to methods of this class.

        :rtype: String
        '''
        return self.coll.name
        
    def query(self, mongoQuery, colNameTuple=(), limit=0, db=None, collection=None, wantMongoId=False):
        '''
        Method for querying the database. The mongoQuery parameter is a 
        dictionary conforming to the MongoDB query conventions. This query
        is passed through to the underlying MongoDB.
        
        The colNameTuple contains a list of field (a.k.a. relational column) names.
        Result documents will contain only those fields. In contrast to MangoDB
        convention, the _id field is not automatically returned in the result dictionaries.
        This field is only included if the caller sets wantMongoId to True

	If colNameTuple is an empty tuple, the entirety of each document is returned 
	for each query result. The rule for the Mongo _id field still holds in this case.
        
        The limit parameter determines how many documents will be returned. A value of zero
        returns the entire result set. A value of 1 makes the method's behavior analogous to
        MangoDB's native find_one() method.
        
        The db and collection keyword arguments allow callers to address a MongoDB database
        and/or collection that are not the default. After the method returns, the default
        database and collection values will still be untouched. 
        
        :param mongoQuery: MangoDB query
        :type mongoQuery: Dict<String,<any>>
        :param colNameTuple: a possibly empty tuple of field/column names to retrieve for each result document
        :type colNameTuple: (String)
        :param limit: maximum number of documents to return
        :type limit: int
        :param db: name of MongoDB database other than the current default
        :type db: String
        :param collection: name of MongoDB collection other than the current default
        :type collection: String
	:param wantMongoId: set to True if return results should include the MongoDb _id field.
	:type wantMongoId: Boolean

        :rtype: {generator<ResultDict>} 
        '''
        with newMongoDB(self, db) as db, newMongoColl(self, collection) as coll:
            # Turn the list of column names to return into
            # Mongo-speak. First, take care of the Python weirdness
            # that turn single-element tuples into the element
            # themselve: ("foo", "bar") ---> ("foo", "bar"). BUT
            #            ("foo") ---> "foo". Whereas
            #            ("foo,") ---> ("foo",). Whereas
            if not isinstance(colNameTuple, tuple):
                colNameTuple = (str(colNameTuple),)
                  
            # Create dict {"colName1" : 1, "colName2" : 1,...} 
            colsToReturn = {}
            for colName in colNameTuple:
                colsToReturn[colName] = 1

            # MongoDB insists on returning the '_id' field, even
            # if you don't ask for it. Suppress that behavior
            # unless wantMongoId is True.
	    if not wantMongoId:
                # Caller did not explicitly ask for the _id field,
                # so suppress _id:
                colsToReturn['_id'] = 0
                
            if len(colsToReturn) > 0:
                cursor = coll.find(mongoQuery, colsToReturn, limit=limit)
            else:
                cursor = coll.find(mongoQuery, limit=limit)
            # Make the cursor findable, so that callers can ask for number of results
            MongoDB.queryCursors[str(mongoQuery)] = cursor
            while True:
                # Termination happens when cursor is exhausted:
                try:
                    yield cursor.next()
                except:
                    # Cursor is exhausted, remove it from our
                    # records:
                    del(MongoDB.queryCursors[str(mongoQuery)])
                    return
    
    def resultCount(self, queryDict):
        '''
        Return number of results in the given query. Only works
        when query has previously been issued via the query()
        method AND at least one result has been extracted. That's
        because the first call to query() only returns a generator.
        This isn't good. 

        :param queryDict: Same query that was provided to the query() method
        :type queryDict: Dict<String,<any>>
        :return: number of results, taking into account limit provided to query(). None if no result has been pulled from query()

        :rtype: {int | None}
        '''
        try:
            cursor = MongoDB.queryCursors[str(queryDict)]
            return cursor.count(with_limit_and_skip=True)
        except KeyError:
            #raise ValueError("Called resultCount() with a query string that was not used in a prior call to query(), or the query results have all been retrieved.")
            return None
    
    def clearCollection(self, db=None, collection=None):
        '''
        Remove all documents from a collection. The affected database/collection
        are the current defaults, if database/collection are None, else the specified
        database/collection is affected.

        :param db: Name of MongoDB database, or None
        :type db: String
        :param collection: Name of MongoDB collection, or None
        :type collection: String
        '''
        with newMongoDB(self, db) as db, newMongoColl(self, collection) as coll:
            coll.remove()
            
    def dropCollection(self, db=None, collection=None):
        '''
        Remove a collection from the database. The affected database/collection
        are the current defaults, if database/collection are None, else the specified
        database/collection is affected.

        :param db: Name of MongoDB database, or None
        :type db: String
        :param collection: Name of MongoDB collection, or None
        :type collection: String
        '''
        with newMongoDB(self, db) as db, newMongoColl(self, collection) as coll:
            coll.drop()
            

    def insert(self, doc_or_docs, db=None, collection=None):
        '''
        Insert the given dictionary into a MongoDB collection.

        :param doc_or_docs: Dictionary whose entries are the documents
        :type doc_or_docs: Dict<String,<any>>
        :param db: Name of MongoDB database, or None
        :type db: String
        :param collection: Name of MongoDB collection, or None
        :type collection: String
        '''
        with newMongoDB(self, db) as db, newMongoColl(self, collection) as coll:
            coll.insert(doc_or_docs)


    def close(self):
        '''
        Release all resources.
        '''
        for cursor in MongoDB.queryCursors.values():
            try:
                cursor.close()
            except:
                pass
        self.client.close()

    # ----------------------------  Private Methods -------------------
    
    def get_db(self):
        '''
        Obtain current default MongoDB database object
        '''
        return self.db
    
    def get_collection(self):
        '''
        Obtain current default MongoDB database object
        '''
        return self.coll
    
    # ----------------------------  Private Classes -------------------    
    
class newMongoDB:
    '''
    Class that enables the construct "with newMongoDB('myDB') as db:"
    See http://effbot.org/zone/python-with-statement.htm for explanation 
    '''
    
    def __init__(self, mongoObj, newDbName):
        self.mongoObj = mongoObj
        self.newDbName = newDbName
        
    def __enter__(self):
        self.savedDBName = self.mongoObj.getDBName()
        if self.newDbName is not None:
            self.mongoObj.setDB(self.newDbName)
        return self.mongoObj.get_db()
        
    def __exit__(self, errType, errValue, errTraceback):
        self.mongoObj.setDB(self.savedDBName)
        # Have any exception re-raised:
        return False
    
    
class newMongoColl:
    '''
    Class that enables the construct "with newMongoColl('myColl') as coll:"
    See http://effbot.org/zone/python-with-statement.htm for explanation 
    '''
    
    def __init__(self, mongoObj, newCollName):
        self.mongoObj = mongoObj
        self.newCollName = newCollName
    
    def __enter__(self):
        self.savedCollName = self.mongoObj.getCollectionName()
        if self.newCollName is not None:
            self.mongoObj.setCollection(self.newCollName)
        return self.mongoObj.get_collection()
        
    def __exit__(self, errType, errValue, errTraceback):
        self.mongoObj.setCollection(self.savedCollName)
        # Have any exception re-raised:
        return False
    
    
    
    



#!/usr/bin/env python
'''
Created on Jan 3, 2014

@author: paepcke

Used by cronRefreshActivityGrade.sh, which
prepares a temp table with information from
courseware_studentmodule (see below for detail).
This script processes the information in that
temp table, and addes the result to table
ActivityGrade. In particular, the following
happens here:

- fill in resource_display_name
- fill in anon_screen_name
- compute the percent_grade column
- parse the original 'state' column's JSON and replace with plusses/minuses


Assumptions:
    o (Optionally) TEMPORARY table StudentmoduleExcerpt holds 
       the result of the following query to courseware_studentmodule:

    	 SET @emptyStr:='';
    	 SET @floatPlaceholder:=-1.0;
    	 SET @intPlaceholder:=-1;
    	 USE edxprod;
    	 CREATE TABLE StudentmoduleExcerpt
    	 SELECT id AS activity_grade_id, \
    	        student_id, \
    	        course_id AS course_display_name, \
    	        grade, \
    	        max_grade, \
    	        @floatPlaceholder AS percent_grade, \
    	        state AS parts_correctness, \
    	        @emptyStr AS answers, \
    	        @intPlaceholder AS num_attempts, \
    	        created AS first_submit, \
    	        modified AS last_submit, \
    	        module_type, \
    	        @emptyStr AS anon_screen_name, \
    	        @emptyStr AS resource_display_name, \
    	        module_id
    	 FROM courseware_studentmodule \
    	 WHERE modified > '"$LATEST_DATE"'; \" \

'''
import argparse
import getpass
import itertools
import os
import re
import sys

from pymysql_utils.pymysql_utils import MySQLDB

# Add json_to_relation source dir to $PATH
# for duration of this execution:
source_dir = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "../json_to_relation/")]
source_dir.extend(sys.path)
sys.path = source_dir
from utils import Utils

class AnonAndModIDAdder(object):

    # Number of rows to process in memory
    # before writing to ActivityGrade:
    BATCH_SIZE = 1000
    
    # For explanation of the following regex patterns,
    # see header comment of parseStateJSON:
    SOLUTION_RESULT_PATTERN  = re.compile(r'[^"]*correctness": "([^"]*)')
    SOLUTION_ANSWERS_PATTERN = re.compile(r'[^:]*: "([^"]*)"')
    
    ACTIVITY_GRADE_COL_NAMES = [
                'activity_grade_id',
                'student_id',
                'course_display_name',
                'grade',
                'max_grade',
                'percent_grade',
                'parts_correctness',
                'answers',
                'num_attempts',
                'first_submit',
                'last_submit',
                'module_type',
                'anon_screen_name',
                'resource_display_name',
                'module_id'
                ]
    
    # Indices into tuples from StudentmoduleExcerpt:
    STUDENT_INT_ID_INDEX = 1
    GRADE_INDEX = 3
    MAX_GRADE_INDEX = 4
    PERCENT_GRADE_INDEX = 5
    PARTS_CORRECTNESS_INDEX = 6
    ANSWERS_INDEX = 7
    NUM_ATTEMPTS_INDEX = 8
    ANON_SCREEN_NAME_INDEX = 12
    RESOURCE_DISPLAY_NAME_INDEX = 13
    MODULE_ID_INDEX = 14
    
    
    def __init__(self, uid, pwd, db='Edx'):
        '''
        ****** Update this comment header
        Make connection to MySQL wrapper.
        @param uid: MySQL user under which to log in. Assumed to be other than None
        @type uid: String
        @param pwd: MySQL password for user uid. May be None.
        @type pwd: {String | None}
        '''
        self.db = db
        if pwd is None:
            self.mysqldbStudModule = MySQLDB(user=uid, db=db)
        else:
            self.mysqldbStudModule = MySQLDB(user=uid, passwd=pwd, db=db)
        # Create a string with the parameters of the SELECT call,
        # (activity_grade_id,student_id,...):
        self.colSpec = AnonAndModIDAdder.ACTIVITY_GRADE_COL_NAMES[0]
        for colName in AnonAndModIDAdder.ACTIVITY_GRADE_COL_NAMES[1:]:
            self.colSpec += ',' + colName
        
        self.pullRowByRow()

    def pullRowByRow(self):
        rowBatch = []
        if self.db == 'unittest':
            queryIt = self.mysqldbStudModule.query("SELECT %s FROM unittest.StudentmoduleExcerpt" % self.colSpec)
        else:
            queryIt = self.mysqldbStudModule.query("SELECT %s FROM edxprod.StudentmoduleExcerpt" % self.colSpec)
        for studmodTuple in queryIt:
            # Results return as tuples, but we need to change tuple items by index.
            # So must convert to list:
            studmodTuple = list(studmodTuple)
            # Resolve the module_id into a human readable resource_display_name:
            moduleID = studmodTuple[AnonAndModIDAdder.MODULE_ID_INDEX]
            studmodTuple[AnonAndModIDAdder.RESOURCE_DISPLAY_NAME_INDEX] = self.getResourceDisplayName(moduleID)
            
            # Compute the anon_screen_name:
            studentIntId = studmodTuple[AnonAndModIDAdder.STUDENT_INT_ID_INDEX]
            try:
                studmodTuple[AnonAndModIDAdder.ANON_SCREEN_NAME_INDEX] = self.getAnonFromIntID(studentIntId)
            except TypeError:
                studmodTuple[AnonAndModIDAdder.ANON_SCREEN_NAME_INDEX] = ''

            # Pick grade and max_grade out of the row,
            # compute the percentage, and place that 
            # back into the row in col 
            grade = studmodTuple[AnonAndModIDAdder.GRADE_INDEX]
            max_grade = studmodTuple[AnonAndModIDAdder.MAX_GRADE_INDEX]
            percent_grade = 'NULL'
            try:
                percent_grade = round((int(grade) * 100.0/ int(max_grade)), 2)
            except:
                pass
            studmodTuple[AnonAndModIDAdder.PERCENT_GRADE_INDEX] = str(percent_grade)

            # Parse 'state' column from JSON and put result into plusses/minusses column:
            (partsCorrectness, answers, numAttempts) = \
                self.parseStateJSON(studmodTuple[AnonAndModIDAdder.PARTS_CORRECTNESS_INDEX])
            
            studmodTuple[AnonAndModIDAdder.PARTS_CORRECTNESS_INDEX] = partsCorrectness
            studmodTuple[AnonAndModIDAdder.ANSWERS_INDEX] = ','.join(answers)
            studmodTuple[AnonAndModIDAdder.NUM_ATTEMPTS_INDEX] = numAttempts
            
            rowBatch.append(studmodTuple)
            if len(rowBatch) >= AnonAndModIDAdder.BATCH_SIZE:
                self.mysqldbStudModule.bulkInsert('ActivityGrade', AnonAndModIDAdder.ACTIVITY_GRADE_COL_NAMES, rowBatch)
                rowBatch = []
        if len(rowBatch) > 0:
            self.mysqldbStudModule.bulkInsert('ActivityGrade', AnonAndModIDAdder.ACTIVITY_GRADE_COL_NAMES, rowBatch)
    
    def getResourceDisplayName(self, moduleID):
        moduleName = Utils.getModuleNameFromID(moduleID)
        return moduleName


    def parseStateJSON(self, jsonStateStr, srcTableName='courseware_studentmodule'):
        '''
        Given the 'state' column from a courseware_studentmodule
        column, return a 3-tuple: (plusMinusStr, answersArray, numAttempts)
        The plusMinusStr will be a string of '+' and '-'. A
        plus means that the problem solution part of an assignment
        submission was correct; a '-' means it was incorrect. The
        plus/minus indicators are arranged in the order of the problem
        subparts; like '++-' for a three-part problem in which the student
        got the first two correct, the last one incorrect.
        
        The answersArray will be an array of answers to the corresponding
        problems, like ['choice_0', 'choice_1'].
        
        Input for a problem solution with two parts looks like this::
            {   		           
    		 "correct_map": {
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_2_1": {
    		     "hint": "",
    		     "hintmode": null,
    		     "correctness": "correct",
    		     "npoints": null,
    		     "msg": "",
    		     "queuestate": null
    		   },
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_3_1": {
    		     "hint": "",
    		     "hintmode": null,
    		     "correctness": "correct",
    		     "npoints": null,
    		     "msg": "",
    		     "queuestate": null
    		   }
    		 },
    		 "input_state": {
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_2_1": {
    		     
    		   },
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_3_1": {
    		     
    		   }
    		 },
    		 "attempts": 3,
    		 "seed": 1,
    		 "done": true,
    		 "student_answers": {
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_2_1": "choice_3",
    		   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_3_1": "choice_0"
    		 }
        }   		        
        
        This structure is ugly enough even when imported into a dict
        via json.loads() that a regular expression solution is faster.
        Three regexp are used:
          - SOLUTION_RESULT_PATTERN  = re.compile(r'[^"]*correctness": "([^"]*)')
              looks for the correctness entries: 'correct', 'incorrect'.
              First the regex throws away front parts the JSON that do not consist
              of 'correctness": '. That's the '"[^"]*correctness": "' par
              of the regex
              Next, a capture group grabs all letters that are not a double
              quote. That's the '([^"]*)' part of the regex. Those capture
              groups will contain the words 'correct' or 'incorrect'.
               
          - SOLUTION_ANSWERS_PATTERN = re.compile(r'[^:]*: "([^"]*)"')
              looks for the answers themselves: 'choice_0', etc. This pattern
              assumes that we first cut off from the JSON all the front part up
              to 'student_answers":'. The regex operates over the rest:
              The '[^:]*: "' skips over all text up to the next colon, followed
              by a space and opening double quote. The capture group grabs the 
              answer, as in 'choice_0'. 
        
        @param jsonStateStr:
        @type jsonStateStr:
        @param srcTableName:
        @type srcTableName:
        @return: plus/minus string, array of participant's answers, number of attempts. 
               If number of attempts is -1 the row was not a problem statement,
               or number of attempts was otherwise unavailable.
        @rtype: (string, [string], int)
        '''
        successResults = ''
        # The following badAnswers array is filled with
        # just the wrong answers. It's maintained, but
        # not currently returned, b/c users didn't feel
        # they needed it.
        badAnswers = [] 
        answers = []
        numAttempts = -1
        
        # Many state entries are not student problem result 
        # submissions, but of the form "{'postion': 4}".
        # Weed those out:
        if jsonStateStr.find('correct_map') == -1:
            #return (successResults, badAnswers, numAttempts)
            return (successResults, answers, numAttempts)
        
        # Get the ['correct','incorrect',...] array;
        # we'll use it later on:
        allSolutionResults = AnonAndModIDAdder.SOLUTION_RESULT_PATTERN.findall(jsonStateStr)
        
        
        # Next, get all the answers themselves.
        # Chop off all the JSON up to 'student_answers":':
        chopTxtMarker = 'student_answers":'
        chopPos = jsonStateStr.find(chopTxtMarker)
        if chopPos == -1:
            # Couldn't find the student answers; fine;
            #return (successResults, badAnswers, numAttempts)
            return (successResults, answers, numAttempts)
        else:
            # Get left with str starting at '{' in
            # "student_answers": {
            #   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_2_1": "choice_3",
            #   "i4x-Medicine-HRP258-problem-8dd11b4339884ab78bc844ce45847141_3_1": "choice_0"
            restJSON = jsonStateStr[chopPos+len(chopTxtMarker):]
            # ... and put the regex to work:
            answers = AnonAndModIDAdder.SOLUTION_ANSWERS_PATTERN.findall(restJSON)
        
        # Find number of attempts:
        # Find '"attempts": 3,...':
        chopTxtMarker = '"attempts": '
        chopPos = jsonStateStr.find(chopTxtMarker)
        if chopPos > 0:
            upToNum = jsonStateStr[chopPos+len(chopTxtMarker):]
            try:
                numAttempts = int("".join(itertools.takewhile(str.isdigit, upToNum)))
            except ValueError:
                # Couldn't find the number of attempts.
                # Just punt.
                pass
            except TypeError:
                # Unicode garbage, clearly not a digit
                pass
                
        # Go through the ['correct','incorrect',...] array,
        # and take two actions: if correct, add a '+' to
        # the successResults str; if 'incorrect' then add
        # a '-' to successResults, and transfer the 'bad'
        # answer to the badAnswers array:
        
        for (i, correctness) in enumerate(allSolutionResults):
            if  correctness == 'correct':
                successResults += '+'
            else:
                successResults += '-'
                try:
                    badAnswers.append(answers[i])
                except IndexError:
                    badAnswers.append('<unknown>')

        #return (successResults, badAnswers, numAttempts)
        return (successResults, answers, numAttempts)
        
    def getAnonFromIntID(self, intStudentId):
        theAnonName = ''
        for anonName in self.mysqldbStudModule.query("SELECT Edx.idInt2Anon(%d)" % intStudentId):
            if anonName is not None:
                # Results come back as tuples; singleton tuple in this case:
                return anonName[0]
            else:
                theAnonName

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]), formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-u', '--user',
                        action='store',
                        help='User ID that is to log into MySQL. Default: the user who is invoking this script.')
    parser.add_argument('-p', '--password',
                        action='store_true',
                        help='request to be asked for pwd for operating MySQL;\n' +\
                             '    default: content of scriptInvokingUser$Home/.ssh/mysql if --user is unspecified,\n' +\
                             '    or, if specified user is root, then the content of scriptInvokingUser$Home/.ssh/mysql_root.')
    parser.add_argument('-w', '--givenPass',
                        dest='givenPass',
                        help='Mysql password. Default: see --password. If both -p and -w are provided, -w is used.'
                        )
    args = parser.parse_args();

    if args.user is None:
        user = getpass.getuser()
    else:
        user = args.user
        
    if args.givenPass is not None:
        pwd = args.givenPass
    else:
        if args.password:
            pwd = getpass.getpass("Enter %s's MySQL password on localhost: " % user)
        else:
            # Try to find pwd in specified user's $HOME/.ssh/mysql
            currUserHomeDir = os.getenv('HOME')
            if currUserHomeDir is None:
                pwd = None
            else:
                # Don't really want the *current* user's homedir,
                # but the one specified in the -u cli arg:
                userHomeDir = os.path.join(os.path.dirname(currUserHomeDir), user)
                try:
                    if user == 'root':
                        with open(os.path.join(currUserHomeDir, '.ssh/mysql_root')) as fd:
                            pwd = fd.readline().strip()
                    else:
                        with open(os.path.join(userHomeDir, '.ssh/mysql')) as fd:
                            pwd = fd.readline().strip()
                except IOError:
                    # No .ssh subdir of user's home, or no mysql inside .ssh:
                    pwd = None
                    
    #************
    #print('UID:'+user)
    #print('PWD:'+str(pwd))
    #sys.exit()
    #************
                    
    anonAdder = AnonAndModIDAdder(user, pwd)



#!/usr/bin/env python

#TODO: In pullLatestLogFiles: show percentage of files downloaded as they are printed.
#TODO: In pullLatestLogFiles: make CLI nice, with commands for pullNew/transform/load
#TODO: In json2relation.py:    if not os.path.exists(logDir):
#                                 os.makedirs(logDir)


'''
Created on Sep 14, 2013

Given a source with JSON structures, derive a schema, and construct 
a relational table. Source can be a local file name, a URL, a
StringIO pseudofile, or a Unix pipe.

JSON structures in the source must be one per line. That is, each line in 
the source must be a self contained JSON object. Pretty printed strings
won't work. 

@author: paepcke

Modifications:
  - Dec 28, 2013: Account copy from Edx to EdxPrivate after load now
                  uses REPLACE INTO, rather than INSERT INTO
  - Dec 29, 2013: changed load time for LoadInfo to local time (was gmtime).
                  Also ensured that the time str entered into LoadInfo
                  is ISO compliant.
'''

from cStringIO import StringIO
from collections import OrderedDict
import copy
import logging
import math
import os
import re
import shutil
import tempfile
import datetime

from col_data_type import ColDataType
from generic_json_parser import GenericJSONParser
from input_source import InputSource, InURI, InString, InMongoDB, InPipe
from output_disposition import OutputDisposition, OutputFile, OutputPipe

class JSONToRelation(object):
    '''
    Given a source with JSON structures, derive a schema, and construct 
    a relational table. Source can be a local file name, a URL, a
    StringIO pseudofile, or a Unix pipe.
    
    JSON structures in the source must be one per line. That is, each line in 
    the source must be a self contained JSON object. Pretty printed strings
    won't work. 
    '''
    
    MAX_SQL_INT = math.pow(2,31) - 1
    MIN_SQL_INT = -math.pow(2,31)
    
    # Regex pattern to check whether a string
    # contains only chars legal in a MySQL identifier
    #, i.e. alphanumeric plus underscore plus dollar sign:
    LEGAL_MYSQL_ATTRIBUTE_PATTERN = re.compile("^[$\w]+$")

    # How long a MySQL packet is allowed to be.
    # MySQL server default is 1M as per documentation,
    # but 16M as per observation. Change that value
    # on server via either  
    #     mysqld --max_allowed_packet=32M
    # or putting this into /etc/mysql/my.cnf
    #       [mysqld]
    #       max_allowed_packet=16M
    # Then change the following constant to the
    # new limit minus about 1K to allow for
    # the INSERT, and column name specs. Or round
    # down like this:
    MAX_ALLOWED_PACKET_SIZE = 1000000; 
    

    # Remember whether logging has been initialized (class var!):
    loggingInitialized = False
    logger = None
        
    def __init__(self, 
                 jsonSource, 
                 destination, 
                 schemaHints=OrderedDict(),
                 jsonParserInstance=None,
                 loggingLevel=logging.INFO,
                 logFile=None,
                 mainTableName='Main',
                 progressEvery=1000):
        '''
        Create a JSON-to-Relation converter. The JSON source can be
        a file with JSON objects, a StringIO.StringIO string pseudo file,
        stdin, or a MongoDB
        
        The destination can be a file, where CSV is written in Excel-readable
        form, stdout, or a MySQL table specification, where the ouput rows
        will be inserted.
        
        SchemaHints optionally specify the SQL types of particular columns.
        By default the processJSONObs() method will be conservative, and
        specify numeric columns as DOUBLE. Even though all encountered values
        for one column could be examined, and a more appropriate type chosen,
        such as INT when only 4-byte integers are ever seen, future additions
        to the table might exceed the INT capacity for that column. Example
        
        If schemaHints is provided, it is a Dict mapping column names to ColDataType.
        The column names in schemaHints must match the corresponding (fully nested)
        key names in the JSON objects::
        
            schemaHints dict: {'msg.length' : ColDataType.INT,
                               'chunkSize' : ColDataType.INT}
        
        For unit testing isolated methods in this class, set jsonSource and
        destination to None.

        This constructor can be thought of as creating the main relational
        table that will hold all results from the JSON parsers in relational
        form. However, parsers may call startNewTable() to build any new tables
        they wish.
        
        :param jsonSource: subclass of InputSource that wraps containing JSON structures, or a URL to such a source
        :type jsonSource: {InPipe | InString | InURI | InMongoDB} (InMongoDB not implemented)
        :param destination: instruction to were resulting rows are to be directed
        :type destination: {OutputPipe | OutputFile }
        :param schemaHints: Dict mapping col names to data types (optional). Affects the default (main) table.
        :type schemaHints: OrderedDict<String,ColDataTYpe>
        :param jsonParserInstance: a parser that takes one JSON string, and returns a CSV row, or other
                                   desired output, like SQL dump statements. Parser also must inform this 
                                   parent object of any generated column names.
        :type jsonParserInstance: {GenericJSONParser | EdXTrackLogJSONParser | CourseraTrackLogJSONParser}
        :param loggingLevel: level at which logging output is show. 
        :type loggingLevel: {logging.DEBUG | logging.WARN | logging.INFO | logging.ERROR | logging.CRITICAL}
        :param logFile: path to file where log is to be written. Default is None: log to stdout.
                        A warning is logged if logFile is None and the destination is OutputPipe. In this
                        case logging messages will be mixed in with the data output
        :type logFile: String
        :param progressEvery: number of JSON object to process before reporting the number in a log info msg. If None, no reporting
        :type  progressEvery: {int | None}
        @raise ValueErrer: when value of jsonParserInstance is neither None, nor an instance of GenericJSONParser,
                        nor one of its subclasses.
        @raise ValueError: when jsonSource is not an instance of InPipe, InString, InURI, or InMongoDB  
        '''

        # If jsonSource and destination are both None,
        # the caller is just unit testing some of the methods
        # below:
        if jsonSource is None and destination is None:
            return
        if not isinstance(jsonSource, InputSource):
            raise ValueError("JSON source must be an instance of InPipe, InString, InURI, or InMongoDB; is %s" % type(jsonSource))
        
        if not isinstance(schemaHints, OrderedDict):
            raise ValueError("The schemaHints, if provided, must be an OrderedDict.")
        
        self.jsonSource = jsonSource
        self.destination = destination
        self.mainTableName = mainTableName
        # Greenwich Mean Time (UTC) as yyyymmddhhssmm
        # self.loadDateTime = time.strftime('%Y%m%d%H%M%s', time.localtime())
        self.loadDateTime = datetime.datetime.now().isoformat()
        self.loadFile = jsonSource.getSourceName()

        # Check schemaHints correctness:
        if schemaHints is not None:
            for typeHint in schemaHints.values():
                if not ColDataType.isinstance(typeHint):
                    raise ValueError("Schema hints must be of type ColDataType")
        self.userDefinedHints = schemaHints

        # The following three instance vars are used for accumulating INSERT
        # values when output is a MySQL dump. 
        # Current table for which insert values are being collected:
        self.currOutTable = None
        
        # Insert values so far (array of value arrays):
        self.currValsArray = []
        
        # Column names for which INSERT values are being collected.
        # Ex.: 'col1,col2':
        self.currInsertSig = None
        
        # Current approximate len of INSERT statement 
        # for cached values:
        self.valsCacheSize = 0;

        # Count JSON objects (i.e. JSON file lines) as they are passed
        # to us for parsing. Used for logging malformed entries:
        self.lineCounter = -1
        
        self.setupLogging(loggingLevel, logFile)

        # Check whether log output would interleave with data output:
        if logFile is None and isinstance(destination, OutputPipe):
            JSONToRelation.logger.warn("If output is to a Unix pipe and no log file name is provided, log output will be mixed with data output.")
        
        if jsonParserInstance is None:
            self.jsonParserInstance = GenericJSONParser(self)
        elif isinstance(jsonParserInstance, GenericJSONParser):
            self.jsonParserInstance = jsonParserInstance
        else:
            raise ValueError("Parameter jsonParserInstance needs to be of class GenericJSONParser, or one of its subclasses.")
        
        #************ Unimplemented Options **************
        #if self.outputFormat == OutputDisposition.OutputFormat.SQL_INSERT_STATEMENTS:
        #    raise NotImplementedError("Output as MySQL statements not yet implemented")
        #*************************************************
        
        # Dict col name to ColumnSpec object:
        self.cols = OrderedDict()
        
        # Position of column for next col name that is
        # newly encountered: 
        self.nextNewColPos = 0;

    def flush(self):
        '''
        MUST be called when no more items are to be converted. Needed
        because for SQL statements, value insertions are held back to
        accumulate multiple insert value for use in a single insert
        statement. This speeds up load times into the databases later.
        If flush() is not called, output may be missing.
        '''
        self.processFinishedRow('FLUSH', self.destination)        

    def setupLogging(self, loggingLevel, logFile):
        if JSONToRelation.loggingInitialized:
            # Remove previous file or console handlers,
            # else we get logging output doubled:
            JSONToRelation.logger.handlers = []
            
        # Set up logging:
        JSONToRelation.logger = logging.getLogger('jsonToRel')
        JSONToRelation.logger.setLevel(loggingLevel)
        # Create file handler if requested:
        if logFile is not None:
            handler = logging.FileHandler(logFile)
        else:
            # Create console handler:
            handler = logging.StreamHandler()
        handler.setLevel(loggingLevel)
#         # create formatter and add it to the handlers
#         formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
#         fh.setFormatter(formatter)
#         ch.setFormatter(formatter)
        # Add the handler to the logger
        JSONToRelation.logger.addHandler(handler)
        #**********************
        #JSONToRelation.logger.info("Info for you")
        #JSONToRelation.logger.warn("Warning for you")
        #JSONToRelation.logger.debug("Debug for you")
        #**********************
        
        JSONToRelation.loggingInitialized = True

    def setParser(self, parserInstance):
        '''
        Set the parser instance to use for upcoming calls to the convert() method.

        :param parserInstance: must be instance of GenericJSONParser or one of its subclasses 
        :type parserInstance: {GenericJSONParser | subclass}
        '''
        self.jsonParserInstance = parserInstance


    def startNewTable(self, tableName, schemaHintsNewTable):
        '''
        Called by parsers when they need to start a new relational
        table beyond the one that is created for them. Ex: edxTrackLogJSONParser.py
        needs auxiliary tables for deeply embedded JSON structures with unpredictable
        numbers of elements (e.g. problem state). The schemaHintsNewTable 
        is an optionally ordered dict. Parsers may add additional columns later
        via calls to ensureColExistence(). Note that for MySQL load files such
        subsequent addition generates ALTER table, which might be expensive. So,
        completeness of the schemaHintsNewTable dict is recommended for MySQL
        load file outputs.

        :param tableName: name by which parser will refer to this table.
        :type tableName: String
        :param schemaHintsNewTable: dict that map column names to SQL types
        :type schemaHintsNewTable: [Ordered]Dict<String,ColumnSpec>
        '''
        self.destination.startNewTable(tableName, schemaHintsNewTable)
        
    def getSourceName(self):
        '''
        Request a human-readable name of the JSON source, which 
        might be a pipe, or a file, or something else. Each input source
        class knows to provide such a name.
        '''
        return self.jsonSource.getSourceName()
        
    def setSchemaHints(self, schemaHints, tableName=None):
        '''
        Given a schema hint dictionary, set the given table's schema
        to the given dictionary. Any schema fragments already defined
        for table tableName will be removed

        :param schemaHints: dict mapping column names to ColumnSpec instances
        :type schemaHints: [Ordered]Dict<String,ColumnSpec>
        :param tableName: name of table whose schema is to be changed. None changes the main (default) table's schema
        :type tableName: String
        '''
        if tableName is None:
            tableName = self.mainTableName
        self.destination.addSchemaHints(tableName, schemaHints)

    def getSchemaHint(self, colName, tableName=None):
        '''
        Given a column name, and a table name, return the ColumnSpec for that column in that table.
        If tableName is None, the table is assumed to be the main (default) table.

        :param colName: name of column whose ColumnSpec instance is wanted
        :type colName: String
        :param tableName: name of table of which the column is a part 
        :type tableName: {String | None}
        :return: a ColumnSpec object that contains the SQL type name of the column

        :rtype: ColumnSpec
        @raise KeyError: if either the table or the column don't exist.  
        '''
        if tableName is None:
            tableName = self.mainTableName
        return self.destination.getSchemaHint(colName, tableName)

    def getSchemaHintByPos(self, pos, tableName=None):
        if tableName is None:
            tableName = self.mainTableName
        return self.destination.getSchemaHintByPos(pos, tableName)
        
    
    def ensureColExistence(self, colName, colDataType, tableName=None):
        if tableName is None:
            tableName = self.mainTableName
        userDefinedHintType = self.userDefinedHints.get(colName, None)
        if userDefinedHintType is not None:
            # We checked type correctness in __init__() method, so trust it here:
            colDataType = userDefinedHintType
        self.destination.ensureColExistence(colName, colDataType, self, tableName)

    def convert(self, prependColHeader=False):
        '''
        Main user-facing API method. Read from the JSON source establish
        in the __init__() call. Create a MySQL schema as the JSON is read.
        Convert each JSON object into the requested output format (e.g. CSV),
        and deliver it to the destination (e.g. a file)

        :param prependColHeader: If true, the final destination, if it is stdout or a file,
                will have the column names prepended. Note that this option requires that
                the output file is first written to a temp file, and then merged with the
                completed column name header row to the final destination that was specified
                by the client. 
        :type prependColHeader: Boolean
        '''
        savedFinalOutDest = None
        if self.destination.getOutputFormat() != self.destination.OutputFormat.SQL_INSERT_STATEMENTS: 
            if prependColHeader:
                savedFinalOutDest = self.destination
                tmpFd  = tempfile.NamedTemporaryFile(suffix='.csv',prefix='jsonToRelationTmp')
                self.tmpFileName = tmpFd.name
                tmpFd.close()
                self.destination = OutputFile(self.tmpFileName, OutputDisposition.OutputFormat.CSV)
                # Need to transfer the table schemas that were added
                # to self.destination before this point:
                savedFinalOutDest.copySchemas(self.destination)

        with self.destination as outFd, self.jsonSource as inFd:
            for jsonStr in inFd:
                # Skip empty rows:
                if jsonStr == '\n' or len(jsonStr) == 0:
                    continue
                newRow = []
                try:
                    # processOneJSONObject will call pushtToTable() for all 
                    # tables necessary for each event type. The method will
                    # direct the top level event information to the table
                    # called self.mainTableName.
                    self.jsonParserInstance.processOneJSONObject(jsonStr, newRow)
                except (ValueError, KeyError) as e:
                    JSONToRelation.logger.warn('Line %s: bad JSON object: %s' % (self.makeFileCitation(), `e`))
                    #***************
                    # Uncomment to get stacktrace for the above caught errors:
                    #import sys
                    #import traceback
                    #traceback.print_tb(sys.exc_info()[2])
                    #***************
                self.bumpLineCounter()

            # Since we hold back SQL insertion values to include them
            # all into one INSERT statement, need to flush after last
            # call to processOneJSONObject:
#             if self.destination.getOutputFormat() == OutputDisposition.OutputFormat.SQL_INSERT_STATEMENTS:
#                 self.processFinishedRow('FLUSH', outFd)
#                 # Give parser a chance to seal the sql file: 
#                 self.jsonParserInstance.finish()
        
            self.processFinishedRow('FLUSH', outFd)
            # Give parser a chance to seal the sql file. If we are 
            # outputting only CSV, and the parser that was used generated MySQL INSERT statements,
            # then let the finish() method know; it will include the CSV load commands, so
            # that the main file can simply be sourced into MySQL:
            if self.destination.getOutputFormat() == OutputDisposition.OutputFormat.CSV:
                self.jsonParserInstance.finish(includeCSVLoadCommands=True, outputDisposition=self.destination)
            else:
                self.jsonParserInstance.finish(includeCSVLoadCommands=False)
        
        
        # If output to other than MySQL table (e.g. CSV file), check whether
        # we are to prepend the column header row:
        if prependColHeader and savedFinalOutDest is not None:
            try:
                #with open(self.destination.name, 'rb') as inFd, OutputDisposition(savedFinalOutDest) as finalOutFd:
                with open(self.destination.name, 'rb') as inFd, savedFinalOutDest as finalOutFd:
                    colHeaders = self.getColHeaders()
                    self.pushToTable(colHeaders, finalOutFd)
                    shutil.copyfileobj(inFd, finalOutFd.fileHandle)
            finally:
                try:
                    # Will fail for String file inputs, b/c strings 
                    # don't get closed:
                    self.tmpFileName.close() # This deletes the file
                except:
                    pass
                

    def pushString(self, whatToWrite):
        '''
        Pushes the given string straight to the output (pipe or file).
        No error checking is done. Used by underlying parsers 
        for SQL other than INSERT statements, such as DROP, CREATE. 

        :param whatToWrite: a complete SQL statement.
        :type whatToWrite: String
        '''
        self.destination.write(whatToWrite)
        
    def pushToTable(self, row, outFd=None):
        '''
        Called both from convert(), and from some parsers to add one row to 
        one table. For CSV outputs the target table is implied: there is only
        one. But for parsers that generate INSERT statements, the targets
        might be different tables on each call. 

        :param row: either an array of CSV values, or a triplet (tableName, insertSig, valsArray) (from MySQL INSERT-generating parsers)
        :type row: {[<any>] | (String, String, [<any>])}
        :param outFd: subclass of OutputDisposition
        :type outFd: subclass of OutputDisposition
        '''
        #****************************
        #if row.count('eventID') > 1:
        #    raise ValueError("Found it!: %s" % self.tmpJSONStr)
        #****************************        
        if outFd is None:
            outFd = self.destination
        self.processFinishedRow(row, outFd)

    def processFinishedRow(self, filledNewRow, outFd):
        '''
        When a row is finished, this method processes the row as per
        the user's disposition. The method writes the row to a CSV
        file, or pipe, or generates an SQL insert statement for later.
        
        The filledNewRow parameter looks different, depending on whether
        the underlying parser generates MySQL insert statement information,
        or CSV rows. The latter are just written to outFD. MySQL insert info
        looks like this: ('tableName', 'insertSig', [valsArray]). The insertSig
        is a string as needed for the INSERT statement's column names part. 
        Ex.: 'col1,col2'. The filledNewRow may also be the string 'FLUSH', which
        triggers flushing all buffers.
        
        :param filledNewRow: the list of values for one row, possibly including empty fields  
        :type filledNewRow: {(String,String,List<<any>>) | List<<any>> | String}
        :param outFd: an instance of a class that writes to the destination
        :type outFd: OutputDisposition
        '''
        # We handle 'rows' destined for MySQL dumps differently
        # from rows destined to CSV. Parsers that generate dump
        # information as they translate JSON provide different
        # information than CSV destined parsers. MySQL dumps provide
        # a list ('tableName', 'insertSig', [valsArray]), while the
        # others produce just an array of values:
        if isinstance(filledNewRow, tuple) or filledNewRow == "FLUSH":
            # Calling parser created INSERT statements:
            filledNewRow = self.prepareMySQLRow(filledNewRow)
        if filledNewRow is not None:
            try:
                outFd.writerow(filledNewRow)
            except Exception as e:
                JSONToRelation.logger.warn('Error during writerow() call in json_to_relation.processFinishRow(): %s' % `e`)

    def prepareMySQLRow(self, insertInfo):
        '''
        Receives either a triple ('tableName', 'insertSig', [valsArray]),
        or the string 'FLUSH'. Generates either None, or a legal MySQL insert statement. 
        The method is lazy.
        
        It collects values to be inserted into the same table, and the same
        columns within that table, and returns a non-Null string only when
        the incoming insertInfo is for a different table, or a different set
        of columns in the same table (as the previous calls). In that case the
        returned string is a legal MySQL INSERT statement, possibly multi-valued.
        
        If insertInfo is the string 'FLUSH', then an INSERT statement is
        returned for any held-back prior values. 
        
        :param insertInfo: information on what to generate for MySQL dumps
        :type insertInfo: {(String, String, [<any>]) | String)}
        '''
        try:
            (tableName, insertSig, valsArray) = insertInfo
        except ValueError:
            if insertInfo == 'FLUSH':
                return self.finalizeInsertStatement()
            else:
                raise ValueError('Bad argument to prepareMySQLRow: %s' % str(insertInfo))

        # Compute approximate new size of hold-back buffer with these new values:
        newCacheSize = self.valsCacheSize + self.calculateHeldBackDataSize(valsArray)
        
        # Have we started accumulating values in an earlier call?
        if self.currInsertSig is not None: 
            if tableName == self.currOutTable and insertSig == self.currInsertSig:
                if newCacheSize > JSONToRelation.MAX_ALLOWED_PACKET_SIZE:
                    # New vals could be held back, but buffer is full:
                    # Construct INSERT statement from the cached values:
                    insertStatement = self.finalizeInsertStatement()
                    # Start accumulating values for this new INSERT request:
                    self.currOutTable = tableName
                    self.currInsertSig = insertSig
                    self.currValsArray.append(copy.copy(valsArray))
                    return insertStatement
                else:
                    # Can hold back the new values:
                    self.valsCacheSize = newCacheSize
                    self.currValsArray.append(copy.copy(valsArray))
                    return None
            else:
                # Were accumulating vals, but this call is for a different
                # table or set of columns:
                insertStatement = self.finalizeInsertStatement()
                # Start accumulating values for this new INSERT request:
                self.currOutTable = tableName
                self.currInsertSig = insertSig
                # Copy the array into the cache buffer: valsArray
                # is passed by reference, and subsequent changes
                # would overwrite the cache. Shallow copy suffices,
                # because all row values are base types:
                self.currValsArray = [copy.copy(valsArray)]
                return insertStatement
            
        # We have not yet started to accumulate values:
        else:
            # If even the first INSERT values are too big for
            # the empty hold-back buffer: just send the INSERT
            # right away:
            if newCacheSize > JSONToRelation.MAX_ALLOWED_PACKET_SIZE:
                valuesPartFileStr = self.constructValuesStr(valsArray)
                insertStatement = "INSERT INTO %s (%s) VALUES %s;" % (tableName, insertSig, valuesPartFileStr.getvalue())
                return insertStatement
            # Hold back the new values:
            self.valsCacheSize = newCacheSize
            self.currOutTable = tableName
            self.currInsertSig = insertSig
            # Copy the array into the cache buffer: valsArray
            # is passed by reference, and subsequent changes
            # would overwrite the cache. Shallow copy suffices,
            # because all row values are base types:
            self.currValsArray = [copy.copy(valsArray)]
            return None
         
    def finalizeInsertStatement(self):
        '''
        Create a possibly multivalued INSERT statement from what is
        stored in self.currOutTable, self.currInsertSig, and self.currValsArray.
        Example return::
        
           INSERT INTO myTable (col2, col2) VALUES
              ('foo',10),
              ('bar',20);

        :return: a fully formed SQL INSERT statement, possibly including multiple values. None if nothing to insert.
        :rtype: {String | None}
        '''
        
        try:
            if len(self.currValsArray) == 0:
                # Nothing to INSERT:
                res = None
                return res
            
            valsFileStr = self.constructValuesStr(self.currValsArray)
            
            res = "INSERT INTO %s (%s) VALUES %s;" % (self.currOutTable, self.currInsertSig, valsFileStr.getvalue())
        finally:
            # We are no longer accumulating INSERT values right now:
            self.currOutTable = None
            self.currInsertSig = None
            self.currValsArray = []
            self.insertValCacheSize = 0
            return res

    def constructValuesStr(self, valsArrays):
        '''
        Takes an array of arrays that hold the values to use in 
        an INSERT statement. Ex: [['foo',10],['bar',20]]. Returns
        a StringIO string file containing a legal INSERT statement's
        VALUES part: ('foo',10),('bar',20)

        :param valsArrays: array of values arrays
        :type valsArrays: [[<any>]]
        :return: IOString string file with legal VALUES section of INSERT statement

        :rtype: IOString
        '''
        # Build the values part:
        valsFileStr = StringIO()
        # Avoid putting anything into the 
        # file string in the nested loop below, so
        # that noting needs to be stripped
        # out afterwards; avoids a string copy.
        # Not sure whether this optimization is
        # needed, but there it is:
        isFirstValTuple = True
        isFirstVal = True
        for insertVals in valsArrays:
            if isFirstValTuple:
                valsFileStr.write('\n    (')
                isFirstValTuple = False
            else:
                valsFileStr.write(',\n    (')
            for insertVal in insertVals:
                if isFirstVal:
                    isFirstVal = False
                else:
                    valsFileStr.write(str(','))
                # Ensure that strings get a quote char arround them, except
                # for 'null', which needs to be written without quotes:
                if insertVal == 'null':
                    valsFileStr.write(insertVal)
                # Turn 'None' entries from JSON-converted empty JSON flds to null: 
                elif insertVal == None:
                    valsFileStr.write('null')
                else:
                    valsFileStr.write("'" + insertVal + "'" if isinstance(insertVal,basestring) else str(insertVal))
            valsFileStr.write(')')
            isFirstVal = True
        return valsFileStr

    def getSchema(self, tableName=None):
        '''
        Returns an ordered list of ColumnSpec instances.
        Each such instance holds column name and SQL type.

        :param tableName: name of table for which schema is wanted. None: the main (default) table. 
        :type tableName: String
        :return: ordered list of column information

        :rtype: (ColumnSpec) 
        '''
        if tableName is None:
            tableName = self.mainTableName
        return self.destination.getSchema(tableName)

    def getColHeaders(self, tableName=None):
        '''
        Returns a list of column header names collected so far.

        :return: list of column headers that were discovered so far by an 
                 associated JSON parser descending into a JSON structure.

        :rtype: [String]
        '''
        headers = []
        for colSpec in self.getSchema(tableName):
            headers.append(colSpec.colName)
        return headers

    def getNextNewColPos(self):
        '''
        Returns the position of the next new column that
        may need to be added when a previously unseen JSON
        label is encountered.

        :return: position in schema where the next new discovered column header is to go.

        :rtype: int 
        '''
        return self.nextNewColPos
    
    def bumpNextNewColPos(self):
        self.nextNewColPos += 1

    def calculateHeldBackDataSize(self, valueArray):
        '''
        Computes number of ASCII bytes needed for the
        values in the given array. Used to estimate 
        when held-back INSERT data needs to be flushed
        to server. Ex: ['foo', 10] returns 7.
        
        :param valueArray: array of mixed-type values to be INSERTed into MySQL at some point
        :type valueArray: [<any>]
        '''
        arrSize = 0
        for val in valueArray:
            arrSize += len(str(val))
        # Add bytes for commas, spaces after the commas, and
        # a pair of quotes around strings (assume worst case of
        # every value being a string:
        arrSize += 4*len(valueArray)
        return arrSize
    
    def ensureLegalIdentifierChars(self, proposedMySQLName):
        '''
        Given a proposed MySQL identifier, such as a column name,
        return a possibly modified name that will be acceptable to
        a MySQL database. MySQL accepts alphanumeric, underscore,
        and dollar sign. Identifiers with other chars must be quoted.
        Quote characters embedded within the identifiers must be
        doubled to be escaped. 

        :param proposedMySQLName: input name
        :type proposedMySQLName: String
        :return: the possibly modified, legal MySQL identifier

        :rtype: String
        '''
        if JSONToRelation.LEGAL_MYSQL_ATTRIBUTE_PATTERN.match(proposedMySQLName) is not None:
            return proposedMySQLName
        # Got an illegal char. Quote the name, doubling any
        # embedded quote chars (sick, sick, sick proposed name): 
        quoteChar = '"'
        if proposedMySQLName.find(quoteChar) > -1:
            quoteChar = "'"
            if proposedMySQLName.find(quoteChar) > -1:
                # Need to double each occurrence of quote char in the proposed name;
                # get a list of the chars, i.e. 'explode' the string into letters:
                charList = list(proposedMySQLName)
                newName = ""
                for letter in charList:
                    if letter == quoteChar:
                        letter = quoteChar + quoteChar
                    newName += letter
                proposedMySQLName = newName
        return quoteChar + proposedMySQLName + quoteChar

    def makeFileCitation(self):
        return self.getSourceName() + ':' + str(self.lineCounter)

    def bumpLineCounter(self):
        self.lineCounter += 1
        
if __name__ == "__main__":
    # Just have this main to test the imports
    print("I ran.")


#!/usr/bin/env python
'''
Created on Feb 12, 2014

@author: paepcke
'''
import argparse
from collections import OrderedDict
import getpass
import os
import sys

from pymysql_utils.pymysql_utils import MySQLDB

from ipToCountry import IpCountryDict


# Add json_to_relation source dir to $PATH
# for duration of this execution:
source_dir = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "../../json_to_relation/json_to_relation/")]
source_dir.extend(sys.path)
sys.path = source_dir



class UserCountryTableCreator(object):

    DEST_TABLE = 'UserCountry'
    
    def __init__(self, user, pwd):
        self.ipCountryXlater = IpCountryDict()
        self.user = user
        self.pwd  = pwd
        self.db = MySQLDB(user=self.user, passwd=self.pwd, db='Edx')
        self.db.dropTable(UserCountryTableCreator.DEST_TABLE)
        self.db.createTable(UserCountryTableCreator.DEST_TABLE, 
                                           OrderedDict({'anon_screen_name' : 'varchar(40) NOT NULL DEFAULT ""',
                                            'two_letter_country' : 'varchar(2) NOT NULL DEFAULT ""',
                                            'three_letter_country' : 'varchar(3) NOT NULL DEFAULT ""',
                                            'country' : 'varchar(255) NOT NULL DEFAULT ""'}))
        
    def fillTable(self):
        values = []
        for (user, ipStr) in self.db.query("SELECT DISTINCT anon_screen_name, ip FROM EventXtract"):
            try:
                (twoLetterCode, threeLetterCode, country) = self.ipCountryXlater.lookupIP(ipStr)
            except (ValueError,TypeError) as e:
                sys.stderr.write("Could not look up one IP string (%s/%s): %s" % (user,ipStr,`e`))
                continue
            values.append(tuple(['%s'%user,'%s'%twoLetterCode,'%s'%threeLetterCode,'%s'%country]))
        
        colNameTuple = ('anon_screen_name','two_letter_country','three_letter_country','country')
        self.db.bulkInsert(UserCountryTableCreator.DEST_TABLE, colNameTuple, values)

    def makeIndex(self):
        self.db.execute("CALL createIndexIfNotExists('UserCountryAnonIdx', 'UserCountry', 'anon_screen_name', 40);")

    def close(self):
        self.db.close()

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]), formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-u', '--user',
                        action='store',
                        help='For load: User ID that is to log into MySQL. Default: the user who is invoking this script.')
    parser.add_argument('-p', '--password',
                        action='store_true',
                        help='For load: request to be asked for pwd for operating MySQL;\n' +\
                             '    default: content of scriptInvokingUser$Home/.ssh/mysql if --user is unspecified,\n' +\
                             '    or, if specified user is root, then the content of scriptInvokingUser$Home/.ssh/mysql_root.'
                        )
    
    args = parser.parse_args();
    if args.user is None:
        user = getpass.getuser()
    else:
        user = args.user
        
    if args.password:
        pwd = getpass.getpass("Enter %s's MySQL password on localhost: " % user)
    else:
        # Try to find pwd in specified user's $HOME/.ssh/mysql
        currUserHomeDir = os.getenv('HOME')
        if currUserHomeDir is None:
            pwd = None
        else:
            # Don't really want the *current* user's homedir,
            # but the one specified in the -u cli arg:
            userHomeDir = os.path.join(os.path.dirname(currUserHomeDir), user)
            try:
                if user == 'root':
                    with open(os.path.join(currUserHomeDir, '.ssh/mysql_root')) as fd:
                        pwd = fd.readline().strip()
                else:
                    with open(os.path.join(userHomeDir, '.ssh/mysql')) as fd:
                        pwd = fd.readline().strip()
            except IOError:
                # No .ssh subdir of user's home, or no mysql inside .ssh:
                pwd = ''
    tblCreator = UserCountryTableCreator(user, pwd)
    tblCreator.fillTable()
    tblCreator.makeIndex()
    tblCreator.close()



'''
Created on Sep 15, 2013

@author: paepcke
'''
from json_to_relation.mongodb import MongoDB
import unittest

TEST_ALL = True

class MongoTest(unittest.TestCase):
    '''
    Test the mongodb.py module. Uses a library that fakes
    a MongoDB server. See https://pypi.python.org/pypi/mongomock/1.0.1
    '''

    def setUp(self):
        self.objs = [{"fname" : "Franco", "lname" : "Corelli"}, 
                     {"fname" : "Leonardo", "lname" : "DaVinci", "age" : 300},
                     {"fname" : "Franco", "lname" : "Gandolpho"}]
        
        self.mongodb = MongoDB(dbName='unittest', collection='unittest')
        self.mongodb.clearCollection(collection="unittest")
        self.mongodb.clearCollection(collection="new_coll")
        self.mongodb.setCollection("unittest")

    def tearDown(self):
        self.mongodb.dropCollection(collection='unittest')
        self.mongodb.dropCollection(collection='new_coll')
        self.mongodb.close()

    @unittest.skipIf(not TEST_ALL, "Skipping")
    def test_update_and_find_one(self):
        self.mongodb.insert(self.objs[0])
        # Get a generator for the results:
        resGen = self.mongodb.query({"fname" : "Franco"}, limit=1, collection="unittest")
        res = resGen.next()
        self.assertEqual('Corelli', res['lname'], "Failed retrieval of single obj; expected '%s' but got '%s'" % ('Corelli', res['lname']))

    @unittest.skipIf(not TEST_ALL, "Skipping")
    def test_set_coll_use_different_coll(self):
        # Insert into unittest:
        self.mongodb.insert(self.objs[0])
        # Switch to new_coll:
        self.mongodb.setCollection('new_coll')
        self.mongodb.insert({"recommendation" : "Hawaii"})
        
        # We're in new_coll; the following should be empty result:
        self.mongodb.query({"fname" : "Franco"}, limit=1)
        resCount = self.mongodb.resultCount({"fname" : "Franco"})
        self.assertIsNone(resCount, "Got non-null result that should be null: %s" % resCount)
        
        # But this search is within new_coll, and should succeed:
        resGen = self.mongodb.query({"recommendation" : {'$regex' : '.*'}}, limit=1)
        res = resGen.next()
        self.assertEqual('Hawaii', res['recommendation'], "Failed retrieval of single obj; expected '%s' but got '%s'" % ('Hawaii', res['recommendation']))
        
        # Try inline collection switch:
        resGen = self.mongodb.query({"fname" : "Franco"}, limit=1, collection="unittest")
        res = resGen.next()
        self.assertEqual('Corelli', res['lname'], "Failed retrieval of single obj; expected '%s' but got '%s'" % ('Corelli', res['lname']))

        # But the default collection should still be new_coll,
        # so a search with unspecified coll should be in new_coll:
        resGen = self.mongodb.query({"recommendation" : {'$regex' : '.*'}}, limit=1)
        res = resGen.next()
        self.assertEqual('Hawaii', res['recommendation'], "Failed retrieval of single obj; expected '%s' but got '%s'" % ('Hawaii', res['recommendation']))

    @unittest.skipIf(not TEST_ALL, "Skipping")
    def test_multi_result(self):
        # Insert two docs with fname == Franco:
        self.mongodb.insert(self.objs[0])
        self.mongodb.insert(self.objs[2])
        resGen = self.mongodb.query({"fname" : "Franco"})
        # To get result count, must retrieve at least one result first:
        resGen.next()
        resCount = self.mongodb.resultCount({"fname" : "Franco"})
        if resCount != 2:
            self.fail("Added two Franco objects, but only %s are found." % str(resCount))
            
    @unittest.skipIf(not TEST_ALL, "Skipping")
    def test_clear_collection(self):
        self.mongodb.insert({"foo" : 10})
        resGen = self.mongodb.query({"foo" : 10}, limit=1)
        res = resGen.next()
        self.assertIsNotNone(res, "Did not find document that was just inserted.")
        self.mongodb.clearCollection()
        resGen = self.mongodb.query({"foo" : 10}, limit=1)
        self.assertRaises(StopIteration, resGen.next)
        
    @unittest.skipIf(not TEST_ALL, "Skipping")
    def test_only_some_return_columns(self):
        # Also tests the suppression of _id col when desired:
        self.mongodb.insert(self.objs[0])
        self.mongodb.insert(self.objs[1])
        resGen = self.mongodb.query({}, ("lname"))
        names = []
        for lnameDict in resGen:
            resCount = self.mongodb.resultCount({})
            self.assertEqual(2, resCount)
            names.append(lnameDict['lname'])
        self.assertItemsEqual(['Corelli','DaVinci'], names, "Did not receive expected lnames: %s" % str(names))


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Jul 2, 2014

@author: paepcke
'''
import StringIO
import os
import re
import shutil
import tempfile
import unittest

from json_to_relation.modulestoreImporter import ModulestoreImporter


TEST_ALL = True

class TestModulestoreImporter(unittest.TestCase):

    # Setup method called once:
    @classmethod
    def setUpClass(cls):
        # Move the existing, full-sized modulestore_latest.json
        # aside, and replace it with a smaller test version:

        # Dir of currently running test script:
        currDir = os.path.dirname(__file__)

        # Where the modulestore_latest.json is stored: 
        TestModulestoreImporter.dataDir = os.path.join(currDir, '..', 'data')
        TestModulestoreImporter.currModStoreLatest = os.path.join(TestModulestoreImporter.dataDir, 'modulestore_latest.json')
        
        # Save current, full-size modulestore_latest.json
        # (preserving its symlink):
        TestModulestoreImporter.savedModStoreLatest = TestModulestoreImporter.currModStoreLatest + '.SAVED'
        try:
            shutil.move(TestModulestoreImporter.currModStoreLatest, TestModulestoreImporter.savedModStoreLatest)
        except IOError:
            pass
        
        # Do the replacement with the test version:
        testModStoreLatest  = os.path.join(currDir, 'data', 'mini_modulestore_latest.json')
        shutil.copy(testModStoreLatest, TestModulestoreImporter.currModStoreLatest)
        
        # The hashLookup.pkl, if it exists, is moved to a save-copy:
        TestModulestoreImporter.hashLookupPicklePath = os.path.join(TestModulestoreImporter.dataDir, 'hashLookup.pkl')
        TestModulestoreImporter.savedHashLookupPicklePath = TestModulestoreImporter.hashLookupPicklePath + '.SAVED'
        if (os.path.exists(TestModulestoreImporter.hashLookupPicklePath)):
            shutil.move(TestModulestoreImporter.hashLookupPicklePath, TestModulestoreImporter.savedHashLookupPicklePath) 

        TestModulestoreImporter.csvTestOutPath = os.path.join(TestModulestoreImporter.dataDir, 'tmpCacheCSVFile.csv') 

    # Teardown method called once:
    @classmethod
    def tearDownClass(cls):
        # Put the true, full size modulestore_latest.json back
        # into its place (preserving its symlink):
        try:
            shutil.move(TestModulestoreImporter.savedModStoreLatest, TestModulestoreImporter.currModStoreLatest)
        except IOError:
            print('NOTE: no modulestore_latest.json found in <projRoot>/json_to_relation/data; run scripts/cronRefreshModuleStore.sh.')
        
        # If hashLookup.pkl existed at the the outset, and was saved,
        # restore it:
        try:
            shutil.move(TestModulestoreImporter.savedHashLookupPicklePath, TestModulestoreImporter.hashLookupPicklePath)
        except:
            pass
        # Remove the csv test output:
        try:
            os.remove(TestModulestoreImporter.csvTestOutPath)
        except:
            pass
         


    def setUp(self):
        jsonFileName = os.path.join(TestModulestoreImporter.dataDir, 'modulestore_latest.json')
        self.importer = ModulestoreImporter(jsonFileName)
        self.pickleCachePath = os.path.join(TestModulestoreImporter.dataDir, 'tmpTestHashLookup.pkl')
        self.uuidRegex = '[a-f0-9]{8}_[a-f0-9]{4}_[a-f0-9]{4}_[a-f0-9]{4}_[a-f0-9]{12}'
        self.pattern   = re.compile(self.uuidRegex)
        self.timestampRegex = r'[1-2][0-9]{3}-[0-1][1-9]-[0-3][0-9]T[0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{0,6}Z{0,1}'
        self.timestampPattern = re.compile(self.timestampRegex)
        # Pattern that recognizes our tmp files. They start with
        # 'oolala', followed by random junk, followed by a period and
        # 0 to 3 file extension letters (always 3, I believe):
        self.tmpFileNamePattern = re.compile('/tmp/oolala[^.]*\..{0,3}')
        
    def tearDown(self):
        # Some of the tests create a temporary
        # pickle file; remove it:
        try:
            os.remove(TestModulestoreImporter.hashLookupPicklePath)
        except:
            pass

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")    
    def testModuleStoreImporter(self):
        self.useTheDict()

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")    
    def testExportHashInfoToCSV(self):

        csvOutFile = TestModulestoreImporter.csvTestOutPath        
        # Write the current cache out to csv:
        self.importer.exportHashInfo(csvOutFile)
        
        csvOutExpected = os.path.join(os.path.dirname(__file__), 'data/modulestoreImporterCsvTruth.csv') 
        self.assertFileContentEquals(csvOutExpected, csvOutFile)
        
        # Check whether dict ops still work:
        self.useTheDict()
        
    def useTheDict(self):
        self.assertEqual('Module One video', self.importer.getDisplayName('c89417444f4443f9a34039be3054962e'))
        self.assertEqual('Medicine', self.importer.getOrg('c89417444f4443f9a34039be3054962e'))
        self.assertEqual('HRP258', self.importer.getCourseShortName('c89417444f4443f9a34039be3054962e'))
        self.assertEqual('video', self.importer.getCategory('c89417444f4443f9a34039be3054962e'))
        self.assertIsNone(self.importer.getRevision('c89417444f4443f9a34039be3054962e'))
        
    #--------------------------------------------------------------------------------------------------    
    
    def assertFileContentEquals(self, expected, filePathOrStrToCompareTo):
        '''
        Compares two file or string contents. First arg is either an open file or a string.
        That is the ground truth to compare to. Second argument is the same: file or string.
        @param expected: the file that contains the ground truth
        @type expected: {File | String }
        @param filePathOrStrToCompareTo: the actual file as constructed by the module being tested
        @type filePathOrStrToCompareTo: { File | String }
        '''
        # Ensure that 'expected' is a File-like object:
        if isinstance(expected, file):
            # It's a file, just use it:
            strFile = expected
        else:
            # Turn into a string 'file':
            with open(expected, 'r') as fd:
                strFile = StringIO.StringIO(fd.read())
        # Check whether filePathOrStrToCompareTo is a file path or a string:
        try:
            open(filePathOrStrToCompareTo, 'r').close()
            filePath = filePathOrStrToCompareTo
        except IOError:
            # We are to compare against a string. Just
            # write it to a tmp file that deletes itself:
            tmpFile = tempfile.NamedTemporaryFile()
            tmpFile.write(filePathOrStrToCompareTo)
            tmpFile.flush()
            filePath = tmpFile.name
        
        with open(filePath, 'r') as fd:
            for fileLine in fd:
                expectedLine = strFile.readline()
                # Some events have one or more UUIDs, which are different with each run.
                # Cut those out of both expected and what we got:
                uuidsRemoved = False
                while not uuidsRemoved:
                    uuidMatch = self.pattern.search(fileLine)
                    if uuidMatch is None:
                        uuidsRemoved = True
                        continue
                    expectedLine = expectedLine[0:uuidMatch.start()] + expectedLine[uuidMatch.end():]
                    fileLine = fileLine[0:uuidMatch.start()] + fileLine[uuidMatch.end():]
                # The timestamp in the LoadInfo table changes on each run,
                # similar to UUIDs:
                timestampMatch = self.timestampPattern.search(fileLine)
                if timestampMatch is not None:
                    expectedLine = expectedLine[0:timestampMatch.start()] + expectedLine[timestampMatch.end():]
                    fileLine = fileLine[0:timestampMatch.start()] + fileLine[timestampMatch.end():]

                # Temp file names also change from run to run.
                # Replace the variable parts (ollalaxxxxx.yyy) with "foo":
                fileLine = self.tmpFileNamePattern.sub("foo", fileLine)
                expectedLine = self.tmpFileNamePattern.sub("foo", expectedLine)
                
                try:
                    self.assertEqual(expectedLine.strip(), fileLine.strip())
                except:
                    #print(expectedLine + '\n' + fileLine)
                    raise
            
            if strFile.readline() != "":
                # expected is longer than what's in the file:
                self.fail("Expected string is longer than content of output file %s" % filePath)


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testModuleStoreImporter']
    unittest.main()


import StringIO
from collections import OrderedDict
import os
import shutil
import tempfile
import unittest

from json_to_relation.col_data_type import ColDataType
from json_to_relation.edxTrackLogJSONParser import EdXTrackLogJSONParser
from json_to_relation.input_source import InputSource, InURI, InString, InMongoDB, InPipe #@UnusedImport
from json_to_relation.json_to_relation import JSONToRelation
from json_to_relation.modulestoreImporter import ModulestoreImporter
from json_to_relation.output_disposition import ColumnSpec, OutputPipe, \
    OutputDisposition, OutputFile


# To speed up modulestore lookups during testing:
#from input_source import InURI
TEST_ALL = True

# The following is very Dangerous: If True, no tests are
# performed. Instead, all the reference files, the ones
# ending with 'Truth.sql' will be changed to be whatever
# the test returns. Use only when you made code changes that
# invalidate all the truth files:
UPDATE_TRUTH = False

class TestJSONToRelation(unittest.TestCase):
    
    def setUp(self):
        super(TestJSONToRelation, self).setUp()
        self.currDir = os.path.dirname(__file__)
        self.tmpLogFile = tempfile.NamedTemporaryFile()
        self.stringSource = InURI(os.path.join(os.path.dirname(__file__),"data/twoJSONRecords.json"))
        self.fileConverter = JSONToRelation(self.stringSource, 
                                            OutputPipe(OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent',
                                            logFile=self.tmpLogFile.name
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        # Remove various test output files if it exists:
        try:
            os.remove("testOutput.csv")
        except:
            pass
        try:
            os.remove("testOutputWithHeader.csv")
        except:
            pass
        try:
            os.remove("testArrays.csv")
        except:
            pass
        try:
            os.remove("testTinyEdXImport.csv")
        except:
            pass
        try:
            os.remove("testEdXImport.csv")
        except:
            pass
        try:
            os.remove("testEdXStressImport.csv")
        except:
            pass
        

        
    def tearDown(self):
        super(TestJSONToRelation, self).tearDown()
        self.tmpLogFile.close()
    
    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_ensure_mysql_identifier_legal(self):

        # Vanilla, legal name
        newName = self.fileConverter.ensureLegalIdentifierChars("foo")
        self.assertEqual("foo", newName)

        # Comma in name requires quoting
        newName = self.fileConverter.ensureLegalIdentifierChars("foo,")
        self.assertEqual('"foo,"', newName)
        
        # Embedded single quotes:
        newName = self.fileConverter.ensureLegalIdentifierChars("fo'o")
        self.assertEqual('"fo\'o"', newName)

        # Embedded double quotes:
        newName = self.fileConverter.ensureLegalIdentifierChars('fo"o')
        self.assertEqual("'fo\"o'", newName)

        # Embedded double and single quotes:
        newName = self.fileConverter.ensureLegalIdentifierChars('fo"o\'bar')
        self.assertEqual("'fo\"o\'\'bar'", newName)

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")        
    def test_simple_json(self):
        # Prints output to display, which we can't catch without
        # fuzzing with stdout. So just ensure no error happens: 
        self.fileConverter.convert()

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_simple_json_to_file(self):
        self.fileConverter = JSONToRelation(self.stringSource, 
                                            OutputFile("testOutput.csv", OutputDisposition.OutputFormat.CSV)
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert()
        with open(os.path.join(self.currDir, 'data/simpleJsonToFileTruth.txt'), 'r') as fd:
            expected = fd.read()
#         expected = "asset,sainani.jpg,HRP258,c4x,Medicine,,image/jpeg,sainani.jpg,262144,/c4x/Medicine/HRP258/asset/sainani.jpg," +\
#                     "22333,,2013-05-08T22:47:09.762Z,,ebcb2a60b0d6b7475c4e9a102b82637b\n" +\
#                     "asset,medstats.png,HRP258,c4x,Medicine,,image/png,medstats.png,262144,/c4x/Medicine/HRP258/asset/medstats.png," +\
#                     "86597,,2013-05-08T22:48:38.174Z,,db47f263ac3532874b8f442ad8937d02"
        if UPDATE_TRUTH:
            self.updateTruthFromFile(os.path.join(self.curDir,'testOutput.csv'), 
                                     os.path.join(self.curDir, 'data/rescueJSONTruth1.txt'))
        else:
            self.assertFileContentEquals(expected, "testOutput.csv")

#     @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
#     def test_json_to_file_with_col_header(self):
#         self.fileConverter = JSONToRelation(self.stringSource, 
#                                             OutputFile("testOutputWithHeader.csv", OutputDisposition.OutputFormat.CSV),
#                                             mainTableName='EdxTrackEvent'
#                                             )
#         edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
#         edxJsonToRelParser.unittesting = True
#         self.fileConverter.jsonParserInstance = edxJsonToRelParser
#         self.fileConverter.convert(prependColHeader=True)
#         expected = '"""_id.category""","""_id.name""","""_id.course""","""_id.tag""","""_id.org""","""_id.revision""",contentType,' +\
#                    'displayname,chunkSize,filename,length,import_path,"""uploadDate.$date""",thumbnail_location,md5\n' +\
#                     "asset,sainani.jpg,HRP258,c4x,Medicine,,image/jpeg,sainani.jpg,262144,/c4x/Medicine/HRP258/asset/sainani.jpg," +\
#                     "22333,,2013-05-08T22:47:09.762Z,,ebcb2a60b0d6b7475c4e9a102b82637b\n" +\
#                     "asset,medstats.png,HRP258,c4x,Medicine,,image/png,medstats.png,262144,/c4x/Medicine/HRP258/asset/medstats.png," +\
#                     "86597,,2013-05-08T22:48:38.174Z,,db47f263ac3532874b8f442ad8937d02"
#         *******self.assertFileContentEquals(expected, "testOutputWithHeader.csv")


    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_arrays(self):
        source = InURI(os.path.join(os.path.dirname(__file__),"data/jsonArray.json"))
        self.fileConverter = JSONToRelation(source, 
                                            OutputFile("testArrays.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent'
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        edxJsonToRelParser.unittesting = True
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert(prependColHeader=True)


    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_embedded_json_strings_comma_escaping(self):
        source = InURI(os.path.join(os.path.dirname(__file__),"data/tinyEdXTrackLog.json"))
        self.fileConverter = JSONToRelation(source, 
                                            OutputFile("testTinyEdXImport.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent'
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert(prependColHeader=True)
    
    
    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_edX_tracking_import(self):
        source = InURI(os.path.join(os.path.dirname(__file__),"data/edxTrackLogSample.json"))
        self.fileConverter = JSONToRelation(source, 
                                            OutputFile("testEdXImport.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent'                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert(prependColHeader=True)

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_edX_stress_import(self):
        source = InURI(os.path.join(os.path.dirname(__file__),"data/tracking.log-20130609.gz"))

        print("Stress test: importing lots...")
        self.fileConverter = JSONToRelation(source, 
                                            OutputFile("testEdXStressImport.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent',
                                            progressEvery=10
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert(prependColHeader=True)
        print("Stress test done")
        

    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def test_schema_hints(self):
        self.fileConverter = JSONToRelation(self.stringSource, 
                                            OutputFile("testOutput.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent',
                                            schemaHints = OrderedDict()
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert()
        schema = self.fileConverter.getSchema()
        #print schema
        #print map(ColumnSpec.getType, schema)
        self.assertEqual(['VARCHAR(40)', 'VARCHAR(40)', 'TEXT', 'VARCHAR(255)', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TEXT', 'DATETIME', 'TEXT', 'DATETIME', 'TEXT', 'TEXT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'INT', 'INT', 'VARCHAR(255)', 'TEXT', 'TEXT', 'TEXT', 'INT', 'TEXT', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TINYINT', 'TEXT', 'VARCHAR(255)', 'INT', 'INT', 'VARCHAR(255)', 'TEXT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'TEXT', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TINYINT', 'TEXT', 'INT', 'INT', 'INT', 'INT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'INT', 'TEXT', 'VARCHAR(40)', 'VARCHAR(40)', 'VARCHAR(40)', 'VARCHAR(40)'],
                         map(ColumnSpec.getType, schema))

        self.stringSource = InURI(os.path.join(os.path.dirname(__file__),"data/twoJSONRecords.json"))
        self.fileConverter = JSONToRelation(self.stringSource, 
                                            OutputFile("testOutput.csv", OutputDisposition.OutputFormat.CSV),
                                            mainTableName='EdxTrackEvent',
                                            schemaHints = OrderedDict({'chunkSize' : ColDataType.INT,
                                                           'length' : ColDataType.INT})
                                            )
        edxJsonToRelParser = EdXTrackLogJSONParser(self.fileConverter, "EdxTrackEvent", useDisplayNameCache=True)
        self.fileConverter.jsonParserInstance = edxJsonToRelParser
        self.fileConverter.convert()
        schema = self.fileConverter.getSchema()
        self.assertEqual(['VARCHAR(40)', 'VARCHAR(40)', 'TEXT', 'VARCHAR(255)', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TEXT', 'DATETIME', 'TEXT', 'DATETIME', 'TEXT', 'TEXT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'INT', 'INT', 'VARCHAR(255)', 'TEXT', 'TEXT', 'TEXT', 'INT', 'TEXT', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TINYINT', 'TEXT', 'VARCHAR(255)', 'INT', 'INT', 'VARCHAR(255)', 'TEXT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'TEXT', 'TEXT', 'VARCHAR(255)', 'TEXT', 'TINYINT', 'TEXT', 'INT', 'INT', 'INT', 'INT', 'VARCHAR(255)', 'VARCHAR(255)', 'VARCHAR(255)', 'INT', 'TEXT', 'VARCHAR(40)', 'VARCHAR(40)', 'VARCHAR(40)', 'VARCHAR(40)'],
                         map(ColumnSpec.getType, schema))


    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def testInsertStatementConstruction(self):
        
        # No value array in hold-back buffer:
        self.fileConverter.currInsertSig = 'col1, col2'
        self.fileConverter.currOutTable = 'TestTable'
        self.fileConverter.currValsArray = []
        self.assertIsNone(self.fileConverter.finalizeInsertStatement())

        # One value array in hold-back buffer:
        self.fileConverter.currInsertSig = 'col1, col2'
        self.fileConverter.currOutTable = 'MyTable'
        self.fileConverter.currValsArray = [['foo', 10]]
        res = self.fileConverter.finalizeInsertStatement()
        #print res
        self.assertEqual("INSERT INTO MyTable (col1, col2) VALUES \n    ('foo',10);", res)
        
        self.fileConverter.currInsertSig = 'col1, col2'
        self.fileConverter.currOutTable = 'MyTable'
        self.fileConverter.currValsArray = [['foo', 10], ['bar', 20]]
        res = self.fileConverter.finalizeInsertStatement()
        #print res
        self.assertEqual("INSERT INTO MyTable (col1, col2) VALUES \n    ('foo',10),\n    ('bar',20);", res)
        
    @unittest.skipIf(not TEST_ALL, "Temporarily disabled")
    def testPrepareMySQLRow(self):
        
        # Pretend to be the edx parser, sending one insert's worth of
        # info. This will fit into the hold-back buffer, and return None:
        insertInfo = ('MyTable', 'col1, col2', [['foo', 10],['bar',20]])
        #print(res)
        self.assertEqual('LoadInfo', self.fileConverter.currOutTable)
        self.assertEqual('load_info_id,load_date_time,load_file', self.fileConverter.currInsertSig)
        currValsArr = self.fileConverter.currValsArray
        self.assertEqual('d4e622ff221b1eec00405f1b69893ff544ac5d75', currValsArr[0][0])
        self.assertEqual('file:///home/paepcke/EclipseWorkspaces/json_to_relation/json_to_relation/test/data/twoJSONRecords.json', currValsArr[0][2])
        
        # Lower the allowed MySQL packet size to force immediate creation of INSERT statement:
        self.fileConverter.__class__.MAX_ALLOWED_PACKET_SIZE = 3
        self.fileConverter.currInsertSig = None
        self.fileConverter.currOutTable = None
        self.fileConverter.currValsArray = []
        res = self.fileConverter.prepareMySQLRow(insertInfo)
        self.assertEqual("INSERT INTO MyTable (col1, col2) VALUES \n    ('foo',10),\n    ('bar',20);", res)

        # Set the allowed MySQL packet size to allow a first INSERT statement to be 
        # held back, but a second call must trigger sending of the held back
        # values, holding back the newly submitted values: 
        
        # First call:
        self.fileConverter.__class__.MAX_ALLOWED_PACKET_SIZE = 211
        self.fileConverter.currInsertSig = None
        self.fileConverter.currOutTable = None
        self.fileConverter.currValsArray = []
        res = self.fileConverter.prepareMySQLRow(insertInfo)
        self.assertIsNone(res)
        self.assertEqual('MyTable', self.fileConverter.currOutTable)
        self.assertEqual('col1, col2', self.fileConverter.currInsertSig)
        self.assertEqual([[['foo', 10], ['bar', 20]]], self.fileConverter.currValsArray)

        # Second call:
        insertMoreInfo = ('MyTable', 'col1, col2', [['blue', 30.1],['green',40.99]])
        res = self.fileConverter.prepareMySQLRow(insertMoreInfo)
        # Should have insert statement for the prior submission...
        self.assertEqual("INSERT INTO MyTable (col1, col2) VALUES \n    (['foo', 10],['bar', 20]);", res)
        # ... and the hold-back buffer should have the new submission:
        self.assertEqual('MyTable', self.fileConverter.currOutTable)
        self.assertEqual('col1, col2', self.fileConverter.currInsertSig)
        self.assertEqual([[['blue', 30.1], ['green', 40.99]]], self.fileConverter.currValsArray)
         
        # Call FLUSH to get the held-back values:
        res = self.fileConverter.prepareMySQLRow('FLUSH')
        self.assertEqual("INSERT INTO MyTable (col1, col2) VALUES \n    (['blue', 30.1],['green', 40.99]);", res)
        self.assertIsNone(self.fileConverter.currOutTable)
        self.assertIsNone(self.fileConverter.currInsertSig)
        self.assertEqual([], self.fileConverter.currValsArray)
        
#--------------------------------------------------------------------------------------------------    
    def assertFileContentEquals(self, expected, filePath):
        strFile = StringIO.StringIO(expected)
        with open(filePath, 'r') as fd:
            for fileLine in fd:
                expectedLine = strFile.readline()
                self.assertEqual(expectedLine.strip(), fileLine.strip())
            
            if strFile.readline() != "":
                # expected is longer than what's in the file:
                self.fail("Expected string is longer than content of output file %s" % filePath)
        
    def updateTruthFromFile(self, referenceFileName, actualTruthFileName):
        '''
        Copy actualTruthFileName content to referenceFileName. 
        ReferenceFileName is a ground truth file to which test run
        results are compared. When the code changes so much that editing
        that ground truth file to match the new correct output of the
        test, then accept the test output as the future ground truth.
        :param referenceFileName: File to be overwritten
        :type referenceFileName: String
        :param actualTruthFileName: Source file
        :type actualTruthFileName: String
        '''
        shutil.copyfile(actualTruthFileName, referenceFileName)
            
if __name__ == "__main__":
    unittest.main()
        


'''
Created on Sep 22, 2013

@author: paepcke
'''
import unittest

from json_to_relation.generic_json_parser import GenericJSONParser, Stack
from json_to_relation.mysqldb import MySQLDB


class Test(unittest.TestCase):


    def testStack(self):
        myStack = Stack()
        myStack.push(3)
        self.assertEqual(3, myStack.pop(), "Bad push/pop of atomic item")
        self.assertEqual(0, myStack.stackHeight(), "Stack not empty after pop")
        self.assertIsNone(myStack.top(), "Top did not return None with empty stack")
        self.assertRaises(ValueError, myStack.top, exceptionOnEmpty=True)
        
        counter = 0
        myStack.push(counter)
        currCounter = myStack.pop()
        currCounter += 10
        myStack.push(currCounter)
        self.assertEqual(10, myStack.top(exceptionOnEmpty=True), "Modifying top of stack did not modify.")
        
    def testRemoveItemPartFromString(self):
        converter = GenericJSONParser(None)
        res = converter.removeItemPartOfString('employee.item.name')
        self.assertEquals('employee.name', res, 'Failed on simple case.')
        
        res = converter.removeItemPartOfString('employee.name')
        self.assertEquals('employee.name', res, "Failed when no 'item' present.")
        
        res = converter.removeItemPartOfString('employee.item')
        self.assertEquals('employee.item', res, "Failed when no trailing 'item', followed by an additional component.")

        res = converter.removeItemPartOfString('item')
        self.assertEquals('item', res, "Failed when only'item' is present.")

        res = converter.removeItemPartOfString('employee.item.lostFound.item.location')
        self.assertEquals('employee.item.lostFound.location', res, "Failed to only remove the last occurrence of 'item'.")
            
        
    def testEnsureMySQLTyping(self):
        mysqlDb = MySQLDB(None,None,None,None,None)
        self.assertEqual('10,"My Poem"', mysqlDb.ensureSQLTyping((10, 'My Poem')))
        self.assertEqual('10,11.23,"My Poem"', mysqlDb.ensureSQLTyping((10, 11.23, 'My Poem')))
        self.assertEqual('10', mysqlDb.ensureSQLTyping((10,)))
        self.assertEqual('"foo"', mysqlDb.ensureSQLTyping(('foo',)))
        
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testStack']
    unittest.main()


#!/usr/bin/env python
'''
Created on Jan 3, 2014

@author: paepcke

Used by cronRefreshGrade.sh.
Accesses a TSV file of the UserGrade table being built. Extracts the screen_name 
column for each TSV row in turn. Computes the corresponding anonymization 
hash, and updates table EdxPrivate.UserGrade's anon_sceen_name column 
in the TSV file with that hash. 
'''
import argparse
import getpass
import os
import sys
import string

# Add json_to_relation source dir to $PATH
# for duration of this execution:
source_dir = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "../json_to_relation/")]
source_dir.extend(sys.path)
sys.path = source_dir

from edxTrackLogJSONParser import EdXTrackLogJSONParser
from mysqldb import MySQLDB

class AnonAdder(object):
    
    def __init__(self, logFile, uid, pwd, tsvFileName, screenNamePos):
        '''
        Make connection to MySQL wrapper.
        @param logFile: file where log entries will be appended.
        @type logFile: String
        @param uid: MySQL user under which to log in. Assumed to be other than None
        @type uid: String
        @param pwd: MySQL password for user uid. May be None.
        @type pwd: {String | None}
        @param tsvFileName: name of TSV file where rows of edxprod's
               certificates_generatedcertificate table are located.
               It is assumed that the caller verified existence and
               readability of this file.
        @type String
        @param screenNamePos: Zero-origin position of the screen name column
               in the TSV file from certificates_generatedcertificate
        @type screenNamePos: int
        '''
        self.uid = uid
        self.pwd = pwd
        self.tsvFileName = tsvFileName
        self.screenNamePos = screenNamePos
        self.logFile = logFile
        
        if pwd is None:
            self.mysqldb = MySQLDB(user=uid, db='EdxPrivate')
        else:
            self.mysqldb = MySQLDB(user=uid, passwd=pwd, db='EdxPrivate')
        
        
    def computeAndAdd(self):
        '''
        The heavy lifting: reads all TSV rows from the certificates_generatedcertificate
        table into memory. the screenNamePos passed into the __init__() method
        is used to find the row's user screen name. That name is hashed
        and appended to the row as a new anon_screen_name column.
        '''
        with open(self.logFile, 'a') as logFd:
            with open(self.tsvFileName, 'r') as tsvFd:
                allRows = tsvFd.readlines()
            for (i, row) in enumerate(allRows[1:]):
                colVals = row.split('\t')
                # Each line's last TSV value element has
                # a \n glued to it. Get rid of that:
                colVals[-1] = colVals[-1].strip()
                # Pick the screen name out of the row:
                try:
                    screenName = colVals[self.screenNamePos]
                except IndexError:
                    logMsg =  "Ill formatted row number %d in user grade file %s: '%s' (dropping this row)\n" % \
                                (int(i), self.tsvFileName, str(row).strip())
                    logFd.write(logMsg)
                    logFd.flush()
                    continue
                # Add the new last element, including
                # the trailing \n:
                colVals.append(EdXTrackLogJSONParser.makeHash(screenName) + '\n')
                # Write the array back into allRows. The '+1'
                # is b/c the enumeration above starts i at 0,
                # which the allRows[1:] starts with the 2nd row,
                # the one after the header:
                allRows[i+1] = string.join(colVals, '\t')
    
            # The first (header column names) row needs to 
            # have the new column appended to it after 
            # again stripping the newline off the last
            # column name, and tagging it onto the 
            # new last col name: 
            colNames = allRows[0].split('\t')
            colNames[-1] = colNames[-1].strip()
            colNames.append('anon_screen_name\n') 
            allRows[0] = string.join(colNames, '\t')
            # Write the new TSV back into the file:
            with open(self.tsvFileName, 'w') as tsvFd:
                tsvFd.writelines(allRows)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]), formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-u', '--user',
                        action='store',
                        help='User ID that is to log into MySQL. Default: the user who is invoking this script.')
    parser.add_argument('-p', '--password',
                        action='store_true',
                        help='request to be asked for pwd for operating MySQL;\n' +\
                             '    default: content of scriptInvokingUser$Home/.ssh/mysql if --user is unspecified,\n' +\
                             '    or, if specified user is root, then the content of scriptInvokingUser$Home/.ssh/mysql_root.')
    parser.add_argument('-w', '--givenPass',
                        dest='givenPass',
                        help='Mysql password. Default: see --password. If both -p and -w are provided, -w is used.'
                        )
    parser.add_argument('-l', '--logFile',
                        dest='logFile',
                        default='/tmp/addAnonToUserGradeTable.log',
                        help='File path to file where log entries are appended. Default: /tmp/addAnonToUserGradeTable.log.'
                        )
    parser.add_argument('tsvFileName',
                        help='File containing the TSV of the certificates_generatedcertificate table obtained from edxprod'
                        )  
    parser.add_argument('screenNameColPos',
                        type=int,
                        help='Zero-origin position of the screen_name column in the TSV of the certificates_generatedcertificate table obtained from edxprod'
                        )  
    args = parser.parse_args();

    if not os.access(args.tsvFileName, os.R_OK):
        print("File %s is not readable or does not exist." % args.tsvFileName)
        parser.print_usage()
        sys.exit()
        
    tsvFileName = args.tsvFileName
    logFile = args.logFile

    if args.screenNameColPos < 0:
        print("Screen name position must be the zero-origin column index of the screen name in the TSV file; was %s" % str(args.screenNameColPos))
        parser.print_usage()
        sys.exit()
    
    screenNameColPos = args.screenNameColPos
        
    if args.user is None:
        user = getpass.getuser()
    else:
        user = args.user
        
    if args.givenPass is not None:
        pwd = args.givenPass
    else:
        if args.password:
            pwd = getpass.getpass("Enter %s's MySQL password on localhost: " % user)
        else:
            # Try to find pwd in specified user's $HOME/.ssh/mysql
            currUserHomeDir = os.getenv('HOME')
            if currUserHomeDir is None:
                pwd = None
            else:
                # Don't really want the *current* user's homedir,
                # but the one specified in the -u cli arg:
                userHomeDir = os.path.join(os.path.dirname(currUserHomeDir), user)
                try:
                    if user == 'root':
                        with open(os.path.join(currUserHomeDir, '.ssh/mysql_root')) as fd:
                            pwd = fd.readline().strip()
                    else:
                        with open(os.path.join(userHomeDir, '.ssh/mysql')) as fd:
                            pwd = fd.readline().strip()
                except IOError:
                    # No .ssh subdir of user's home, or no mysql inside .ssh:
                    pwd = None
                    
    #************
    #print('UID:'+user)
    #print('PWD:'+str(pwd))
    #sys.exit()
    #************
                    
    anonAdder = AnonAdder(logFile, user, pwd, tsvFileName, screenNameColPos)
    anonAdder.computeAndAdd()
    



#!/usr/bin/env python

import argparse
import datetime
import os
import re
import socket
import sys
import time

# Add json_to_relation source dir to $PATH
# for duration of this execution:
source_dir = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "../json_to_relation/")]
source_dir.extend(sys.path)
sys.path = source_dir

from edxTrackLogJSONParser import EdXTrackLogJSONParser
from input_source import InURI
from json_to_relation import JSONToRelation
from output_disposition import OutputDisposition, OutputFile


# Transforms a single .json OpenEdX tracking log file to
# relational tables. See argparse below for options.

# For non-Stanford installations, customize the following
# setting of LOCAL_LOG_STORE_ROOT. This is the file system 
# directory from which S3 key paths descend. Example: Stanford's
# S3 file keys where .json tracking logs are stored are of the form
#    tracking/app10/tracking.log-<date>.gz
# For transform to relational tables we copy these files
# to /foo/bar/, so that they end up like this:
# /foo/bar/tracking/app10/tracking.log-<date>.gz
# LOCAL_LOG_STORE_ROOT in this example is /foo/bar/

hostname = socket.gethostname()
if hostname == 'duo':
    LOCAL_LOG_STORE_ROOT = "/home/paepcke/Project/VPOL/Data/EdX/EdXTrackingLogsTests/"                            
elif hostname == 'mono':
    LOCAL_LOG_STORE_ROOT = "/home/paepcke/Project/VPOL/Data/EdXTrackingOct22_2013/"
elif hostname == 'datastage':
    LOCAL_LOG_STORE_ROOT = "/home/dataman/Data/EdX"

def buildOutputFileName(inFilePath, destDir, fileStamp):
    '''
    Given the full path to a .json tracking log file, a destination
    directory where results of a transform to relational tables will
    go, and a timestamp to ensure uniqueness, generate a new .sql
    filename that will be used by the transform. Example: given::
      /EdX/EdXTrackingLogsTests/tracking/app10/tracking.log-20130610.gz
    and given destDir of /tmp, and LOCAL_LOG_STORE_ROOT being /EdX/EdXTrackingLogsTests/
    /returns something like::
      /tmp/racking.app10.tracking.log-20130610.gz.2013-12-05T00_33_10.462711_5050.sql
    
    That is the LOCAL_STORE_ROOT is removed from the infile, leaving just
    tracking/app10/tracking.log-20130610.gz. Then the destDir is prepended,
    the fileStamp is appended, together with a trailing .sql
     
    @param inFilePath: full path to .json file
    @type inFilePath: String
    @param destDir: full path to destination directory
    @type destDir: String
    @param fileStamp: timestamp
    @type fileStamp: String
    @return: a full filename with a .sql extension, derived from the input file name
    @rtype: String
    '''
    rootEnd = inFilePath.find(LOCAL_LOG_STORE_ROOT)
    if rootEnd < 0:
        return os.path.join(destDir, os.path.basename(inFilePath)) + '.' + fileStamp + '.sql'
    subTreePath = inFilePath[rootEnd+len(LOCAL_LOG_STORE_ROOT):]
    subTreePath = re.sub('/', '.', subTreePath)
    if subTreePath[0] == '.':
        subTreePath = subTreePath[1:]
    res = os.path.join(destDir, subTreePath + '.' + fileStamp + '.sql')
    return res


if __name__ == "__main__":

    parser = argparse.ArgumentParser(prog='json2sql.py')
    parser.add_argument('-x', '--expungeTables',
                        help='DROP all tables in database before beginning transform',
                        dest='dropTables',
                        action='store_true',
                        default=False)
    # parser.add_argument('-l', '--logFile', 
    #                     help='fully qualified log file name. Default: no logging.',
    #                     dest='logFile',
    #                     default='/tmp/j2s.sql');
    parser.add_argument('-v', '--verbose', 
                        help='print operational info to console.', 
                        dest='verbose',
                        action='store_true');
    parser.add_argument('-t', '--targetFormat', 
                        help='Output one CSV file per table, a dump file as would be created my mysqldump, or both. Default: sql_dump', 
                        dest='targetFormat',
                        default='sql_dump',
                        choices = ['csv', 'sql_dump', 'sql_dump_and_csv']);
    parser.add_argument('destDir',
                        help='file path for the destination .sql/csv file(s)')                        
    parser.add_argument('inFilePath',
                        help='json file path to be converted to sql/csv.') 
    
    
    args = parser.parse_args();

    # Output file is name of input file with the
    # .json extension replaced by .sql, and a unique
    # timestamp/pid added to avoid name collisions during
    # parallel processing:
    dt = datetime.datetime.fromtimestamp(time.time())
    fileStamp = dt.isoformat().replace(':','_') + '_' + str(os.getpid())

    outFullPath = buildOutputFileName(args.inFilePath, args.destDir, fileStamp) 

    #********************
    #print('In: %s' % args.inFilePath)
    #print('Out: %s' % outFullPath)
    #sys.exit()
    #********************

    # Log file will go to <destDir>/../TransformLogs, the file being named j2s_<inputFileName>.log:
    logDir = os.path.join(args.destDir, '..') + '/TransformLogs'
    if not os.access(logDir, os.W_OK):
        try:
            os.makedirs(logDir)
        except OSError:
            # Log dir already existed:
            pass

    logFile = os.path.join(logDir, 'j2s_%s_%s.log' % (os.path.basename(args.inFilePath), fileStamp))
    

#    print('xpunge: %s' % args.dropTables)
#    print('verbose: %s' % args.verbose)
#    print('destDir: %s' % args.destDir)
#    print('in=FilePath: %s' % args.inFilePath)
#    print('outFullPath: %s' % outFullPath)
#    print('logFile: %s' % logFile)

    # Create an instance of JSONToRelation, taking input from the given file:
    # and pumping output to the given output path:

    if args.targetFormat == 'csv':
        outputFormat = OutputDisposition.OutputFormat.CSV
    elif args.targetFormat == 'sql_dump':
        outputFormat = OutputDisposition.OutputFormat.SQL_INSERT_STATEMENTS
    else:
        outputFormat = OutputDisposition.OutputFormat.SQL_INSERTS_AND_CSV

    outSQLFile = OutputFile(outFullPath, outputFormat, options='wb')  # overwrite any sql file that's there
    jsonConverter = JSONToRelation(InURI(args.inFilePath),
                                   outSQLFile,
                                   mainTableName='EdxTrackEvent',
    				               logFile=logFile
                                   )
    try:
        jsonConverter.setParser(EdXTrackLogJSONParser(jsonConverter, 
        						  'EdxTrackEvent', 
        						  replaceTables=args.dropTables, 
        						  dbName='Edx'
        						  ))
    except Exception as e:
        with open(logFile, 'w') as fd:
            fd.write("In json2sql: could not create EdXTrackLogJSONParser: %s" % `e`)
        # Try to delete the .sql file that was created when 
        # the OutputFile instance was made in the JSONToRelation
        # instantiation statement above:
        try:
            outSQLFile.remove();
        except Exception as e:
            pass
        sys.exit(1)
        
    jsonConverter.convert()

